var $4 = {
	// Create new HTML Element with options
	// @param {_} - First is Tag name <) console.dir($4.cr("option","value","email","textContent","Hello")); 
	// @return {HTMLElement}
	cr: function(){
		if(!arguments[0]){
			throw("Don't set Tag name");
		}
		var EL = document.createElement(arguments[0]);
		for(var i = 1, m = arguments.length; i < m; i += 2){
			arguments[i] && arguments[i + 1] && (EL[arguments[i]] = arguments[i + 1]);
		}
		return EL;
	},
	parentByTag: function(node, tagName){
		tagName = tagName.toUpperCase();
		var currentNode = node;
		
		while(currentNode.tagName != tagName && currentNode != document.body){
			currentNode = currentNode.parentNode;
		}
		
		return currentNode != document.body ? currentNode : undefined;
	},
	removeNode: function(node){
		node.parentNode.removeChild(node);
	},
	emptyNode: function(node){
		var childNodes = node.childNodes;
		for(i = childNodes.length; i--;){
			node.removeChild(childNodes[i]);
		}
		return node;
	},
	removeNode: function(node){
		node && node.parentNode && node.parentNode.removeChild(node);
	},
	// set Data attribute to node
	// @param {HtmleElement} node
	// @param {String} field 
	// @param {String} value
	setDataValue: function(node, field, value){
		var 	attrField = 'data-' + field.replace(/([A-Z])/g, function(str,p){return '-' + p.toLowerCase();}),
				dataField = field.replace(/-(\w)/g, function(str,p){return p.toUpperCase();});	

		node.dataset[dataField] = value;
		node.setAttribute(attrField,value);
		return node;
	},
};

// Storage for settings
var SettingsStorage = function(config){
	Backside.Model.call(this, config);
}
Backside.extend(SettingsStorage, Backside.Model);

SettingsStorage.prototype.setNoProtection = function(){
	this.set('analytics_enabled', false);
	this.set('social_enabled', false);
	this.set('track_enabled', false);
	self.port.emit('updateSettings', {
		analytics_enabled: false,
		social_enabled: false,
		track_enabled: false
	});
};
SettingsStorage.prototype.setLowLevelProtection = function(){
	this.set('analytics_enabled', false);
	this.set('social_enabled', false);
	this.set('track_enabled', true);
	self.port.emit('updateSettings', {
		analytics_enabled: false,
		social_enabled: false,
		track_enabled: true
	});
};
SettingsStorage.prototype.setMediumLevelProtection = function(){
	this.set('analytics_enabled', true);
	this.set('social_enabled', false);
	this.set('track_enabled', true);
	self.port.emit('updateSettings', {
		analytics_enabled: true,
		social_enabled: false,
		track_enabled: true
	});
};
SettingsStorage.prototype.setHightLevelProtection = function(){
	this.set('analytics_enabled', true);
	this.set('social_enabled', true);
	this.set('track_enabled', true);
	self.port.emit('updateSettings', {
		analytics_enabled: true,
		social_enabled: true,
		track_enabled: true
	});
};
SettingsStorage.prototype.updateDB = function(){
	self.port.emit('updateDB');
};

SettingsStorage.prototype.updateFileCheckSettings = function(conf){
	if(conf) {
		this.set('scan_timeout_ms', conf.scan_timeout_ms);
		this.set('heuristic_analysis', conf.heuristic_analysis);
		this.set('packer_max_level', conf.packer_max_level);
		this.set('archive_max_level', conf.archive_max_level);
		this.set('container_max_level', conf.container_max_level);
		this.set('max_compression_ratio', conf.max_compression_ratio);
		this.set('mail_max_level', conf.mail_max_level);
	}
	self.port.emit('updateFileCheckSettings', {
		scan_timeout_ms: conf.scan_timeout_ms,
		heuristic_analysis: conf.heuristic_analysis,
		packer_max_level: conf.packer_max_level,
		archive_max_level: conf.archive_max_level,
		container_max_level: conf.container_max_level,
		max_compression_ratio: conf.max_compression_ratio,
		mail_max_level: conf.mail_max_level
	});
};

var 	Settings = new SettingsStorage(),
		FlashModel = new Backside.Model(),
		AdsModel = new Backside.Model(),
		i18n = {};

FlashModel.on('change:enabled', function(value, name, model, oldValue){
	self.port.emit('updateSettings', {
		flash_enabled: value
	});
});
AdsModel.on('change:enabled', function(value, name, model, oldValue){
	self.port.emit('updateSettings', {
		ads_enabled: value
	});

});
FlashModel.id = 'flash';
AdsModel.id = 'ads';

FlashModel.saveExceptions = AdsModel.saveExceptions = function(){
	var obj = {};

	obj[this.id == 'flash' ? 'flash_exceptions' : 'ads_exceptions'] = this.get('exceptions');
	self.port.emit('updateSettings', obj);
};

/* BlockView ******************************************************/
var BlockView = function(config){
	Backside.View.call(this, config);
}
Backside.extend(BlockView, Backside.View);
BlockView.prototype.tagName = 'form'; // tagName
BlockView.prototype.className = 'optpage_block'; // tagName
BlockView.prototype.initialize = function(config){
	Backside.View.prototype.initialize.call(this, config);

	this.controls.btn.setAttribute('data-count', this.model.get('exceptions').length);
	this.controls.select.value = !this.model.get('enabled') ? 'allow' : 'deny';

	this.model.on('change:exceptions', function(item, model){
		this.controls.btn.setAttribute('data-count', model.get('exceptions').length);
	}.bind(this));

	this.el.onsubmit = function(e){
		e.preventDefault();
		var popupView = new PopupView({
			model: config.model
		});
		config.heap.appendChild(popupView.el);
		popupView.open();
	};
	this.controls.select.onchange = function(e){
		config.model.change('enabled', e.target.value != 'allow');
	};

};
BlockView.prototype.template = 
	'label.optpage_block-label$label, select$select.optpage_block-select >' +
		'option@value=allow$allowOption,' +
		'option@value=deny$denyOption <<' +
	'button.optpage_block-btn@type=submit$btn@data-count=0';
BlockView.prototype.render = function(){
	var fragment = new Fragment(this.template);

	for(var key in fragment.alias){
		this.controls[key] = fragment.alias[key];
	}

	this.controls.label.textContent = this.model.get('labelText');
	this.controls.allowOption.textContent = this.model.get('allowText');
	this.controls.denyOption.textContent = this.model.get('denyText');
	this.controls.btn.textContent = this.model.get('btnText');
	this.el.appendChild(fragment.getNode());
};

/* PopupView ******************************************************/
var PopupView = function(config){
	Backside.View.call(this, config);
	this.el.style.display = 'none';
	this.cancelList = [];
	this.addList = [];
}
Backside.extend(PopupView, Backside.View);
PopupView.prototype.className = 'optpage_popup-wrap';
PopupView.prototype.fragTemp =
		'div.optpage_popup-column$background > div.optpage_popup-content$content > ' +
			'button.closePopup$close, ' +
			'h2.optpage_title1 > {{popupTitle}} < ' +
			'.optpage_liter1.optpage_mb25 > {{popupDescription}} < ' +
			'.optpage_urlblock-wrap > '+
				'input@type=text$input.optpage_urlblock-input , ' +
				'select.optpage_urlblock-select@name=addingFlashSiteUrlSelector$select > '+
					'option@value=allow > {{tAllow}} < ' +
					'option@value=deny > {{tDeny}} << ' +
				'button.optpage_urlblock-btn$add <' +
			'.optpage_ribtable-wrap > '+
				'table.optpage_ribtable >' +
					'tbody$table <<' +
			'.optpage_button-wrap > button.optpage_btn.__blue$save > {{ready}}';
PopupView.prototype.render = function(){
	var fragment = new Fragment(Backside._.supplant(this.fragTemp, this.model.attr));

	for(var key in fragment.alias){
		this.controls[key] = fragment.alias[key];
	}
	this.el.appendChild(fragment.getNode());
};
// @param {String} nationalURL - url in national representation
PopupView.prototype.addRow = function(allow, id, baseURL, nationalURL){
	var 	$tr = $4.cr('tr', 'id', id),
			$td1 = $4.cr('td'),
			$td2 = $4.cr('td'),
			$td3 = $4.cr('td', 'className', 'optpage_ribtable_btn-cell'),
			$cell1 = $4.cr('div', 'className', 'optpage_ribtable_max-column', 'title', baseURL, 'textContent', nationalURL || baseURL),
			$cell3 = $4.cr('div', 'className', 'removesite'),
			$select = $4.cr('select', 'className', 'styled-select'),
			$option1 = $4.cr('option', 'value', 'allow', 'textContent', this.model.get('tAllow')),
			$option2 = $4.cr('option', 'value', 'deny', 'textContent', this.model.get('tDeny'));


	$4.setDataValue($cell3, 'id', id);
	$4.setDataValue($select, 'id', id);
	(allow ? $option1 : $option2).setAttribute('selected', true);
	
	this.controls.table.appendChild($tr);
	$tr.appendChild($td1);
	$tr.appendChild($td2);
	$tr.appendChild($td3);
	$td1.appendChild($cell1);
	$td3.appendChild($cell3);
	$td2.appendChild($select);
	$select.appendChild($option1);
	$select.appendChild($option2);

	// TODO неплохо было бы почистить baseURL от html entities 
	// 	this.supplant(this.itemTemplate, {
	// 		id: id,
	// 		url: baseURL,
	// 		targetUrl: nationalURL || baseURL, 
	// 		allowAttr: (allow ? "selected" : ""),
	// 		allowLiter: this.model.get('tAllow'),
	// 		denyAttr: (!allow ? "selected" : ""),
	// 		denyLiter: this.model.get('tDeny'),
	// 	})

};
PopupView.prototype._DOMAIN_REG = /^(?:https?:\/\/)?([^\/?:#]*)/i;
// @param {Array} exceptions
PopupView.prototype.getDomain = function(urlStr){
	var parseDomain = this._DOMAIN_REG.exec(urlStr);
	return Array.isArray(parseDomain) && parseDomain[1];
};
PopupView.prototype.containsNonLatinSymbols = function(str){
	return /[^.\-_a-z\w]/.test(str);
};
// @param {Array} exceptions
PopupView.prototype.addCollection = function(exceptions){
	var 	nationalURL,
			baseURL;
			
    for(var i = 0; i < exceptions.length; i++){
		baseURL = exceptions[i].baseURL;
		nationalURL = baseURL;
			
		if(this.containsNonLatinSymbols(baseURL)){ // maybe national domain
			baseURL = punycode.toASCII(baseURL);
		}else if(baseURL.indexOf('xn--') > -1){
			nationalURL = punycode.toUnicode(baseURL);
		}
			
		this.addRow(exceptions[i].allow, exceptions[i].id, baseURL, nationalURL);
	}
};
// detect exceptions by baseURL field
PopupView.prototype.exceptionExist= function(url){
	var 	exceptions = this.model.get('exceptions'),
			index = exceptions.length,
			excHost;
			
	while(index--){
		// excHost = exceptions[index].baseURL;
		
		//// hot fix:
		//// if we save Cyrillic domain (in popup) transform it to correct! Also need add puny library at popup...
		// if(this.containsNonLatinSymbols(excHost)){ // maybe national domain
			// exceptions[index].baseURL = punycode.toASCII(excHost);
		// }
		if(exceptions[index].baseURL == url){
			return true;
		}
	}
	return false;
};
PopupView.prototype.addException = function(){
	var 	url = this.controls.input.value,
			allow = this.controls.select.value == 'allow',
			id = ("uni" + Math.random()).replace(".", ""),
			baseURL = (this.getDomain(url) || '').replace(/^www\./, ''),
			nationalURL;
	
	if(!baseURL){
		return;
	}

	// FIX special for FFox (convert puny code to unicode)
	if(baseURL.indexOf('xn--') > -1){
		baseURL = punycode.toUnicode(baseURL);
	}
	nationalURL = baseURL;

	// if(this.containsNonLatinSymbols(baseURL)){ // maybe national domain
	// 	baseURL = punycode.toASCII(baseURL);
	// }else if(baseURL.indexOf('xn--') > -1){
	// 	nationalURL = punycode.toUnicode(baseURL);
	// }
	
	if(!this.exceptionExist(baseURL)){
		this.addRow(allow, id, baseURL, nationalURL);

		var exceptions = this.model.get('exceptions');
		exceptions.push({
			id: id,
			baseURL: baseURL,
			allow: allow,
			content: false
		});
		this.model.change('exceptions', exceptions);
	}
	this.controls.input.value = '';
	this.cancelList.push(id);
};
PopupView.prototype.initialize = function(config){
	Backside.View.prototype.initialize.call(this, config);
	var _popup = this;
	
	this.on('add', 'click', function(){
		this.addException();
	}.bind(this));
	this.on('input', 'keydown', function(e){
		if(e.keyCode == 13){
			this.addException();
		}
	}.bind(this));
	this.on('save', 'click', function(){
		_popup.close();
	});

	var cancelHandler = function(){
		var 	exceptions = this.model.get('exceptions'),
				i = exceptions.length;

		while(i-- > 0){
			if(this.cancelList.indexOf(exceptions[i].id) > -1){
				exceptions.splice(i, 1);
			}
		}

		i = this.addList.length;
		while(i-- > 0){
			if(this.cancelList.indexOf(this.addList[i].id) == -1){
				exceptions.push(this.addList[i]);
			}
		}
		this.model.change('exceptions', exceptions);
	
		_popup.close();
	}.bind(this);
	this.on('close', 'click', cancelHandler);
	this.on('background', 'click', cancelHandler);
	this.on('content', 'click', function(e){
		e.stopPropagation();
	});
	this.on('table', 'click', function(e){
		var target = e.target;
		
		if(target.className == 'removesite'){
			var 	exceptionId = target.dataset.id,
					tr = $4.parentByTag(target, 'tr');
			var 	exceptions = this.model.get('exceptions'),
					i = exceptions.length;
							
			tr && $4.removeNode(tr);

			while(i-- > 0){
				if(exceptions[i].id == exceptionId){
					this.addList.push(exceptions[i]);
					exceptions.splice(i, 1);
				}
			}
			this.model.change('exceptions', exceptions);
		}
	}.bind(this));
	this.on('table', 'change', function(e){
		var target = e.target;
		var exceptionId = target.dataset.id;
		
		if(exceptionId){
			// TODO refactor this code
			var 	list = this.model.get('exceptions'),
					index = list.length,
					pos;
					
			while(index--){
				if(list[index].id == exceptionId){
					pos = index;
					break;
				}
			}
			
			if(pos != undefined){
				this.model.set('exceptions.' + pos + '.allow', target.value == 'allow');
				// We changed property of array with listener
				// So we haven't eventType checkeing at handler and don't use change
				// this.model.change('exceptions.0.allow', target.value == 'allow');
			}
		}
	}.bind(this));
};
PopupView.prototype.open = function(){
	document.body.style.overflow = 'hidden';
	this.el.style.display = '';
	this.addCollection(this.model.get('exceptions'));
},
PopupView.prototype.close = function(){
	this.model.saveExceptions();
	this.destroy();
	this.el.parentNode.removeChild(this.el);
	document.body.style.overflow = '';
};

/* OptionView ******************************************************/
var OptionView = function(config){
	Backside.View.call(this, config);
}
Backside.extend(OptionView, Backside.View);

OptionView.prototype._withZero = function(n){
	return (n < 10 ? '0' : '') + n;
};
OptionView.prototype.renderDate = function(timeStamp){
	var 	date = new Date(timeStamp),
			day = this._withZero(date.getDate()),
			month = this._withZero(date.getMonth() + 1),
			year =  date.getFullYear(),
			min = this._withZero(date.getMinutes()),
			hour = this._withZero(date.getHours()),
			sec = this._withZero(date.getSeconds());
			
	this.controls.updateDay.textContent = day + '.' + month + '.' + year;
	this.controls.updateTime.textContent = hour + ':' + min + ':' + sec;
};
OptionView.prototype.initialize = function(config){
	Backside.View.prototype.initialize.call(this, config);
	
	this.on('tracksNo', 'change', function(e){
		var val = e.target.checked;
		Settings.setNoProtection();
	});
	this.on('tracksMin', 'change', function(e){
		var val = e.target.checked;
		Settings.setLowLevelProtection();
	});
	this.on('tracksMid', 'change', function(e){
		var val = e.target.checked;
		Settings.setMediumLevelProtection();
	});
	this.on('tracksMax', 'change', function(e){
		var val = e.target.checked;
		Settings.setHightLevelProtection();
	});
	
	var flashBlockSubView = new BlockView({
		model: FlashModel,
		heap: this.controls.heap,
	});
	var adsBlockSubView = new BlockView({
		model: AdsModel,
		heap: this.controls.heap,
	});

	this.controls.tBlockContent.appendChild(flashBlockSubView.el);
	this.controls.tBlockContent.appendChild(adsBlockSubView.el);
	
	if(Settings.get('social_enabled')){
		this.controls.tracksMax.checked = true;
	}else if(Settings.get('analytics_enabled')){
		this.controls.tracksMid.checked = true;
	}else if(Settings.get('track_enabled')){
		this.controls.tracksMin.checked = true;
	}else{
		this.controls.tracksNo.checked = true;
	}
	
	var 	timestamp = Settings.get('updateTimeStamp'),
			_btnLock = false,
			_updateTimer;

	this.renderDate(timestamp);
	this.controls.dbVersion.textContent = Settings.get('version');
	Settings.on('change:updateTimeStamp', function(value, name, model, oldValue){
		this.renderDate(value);
		_btnLock = false; // leave lock from update btn
	}.bind(this));
	Settings.on('change:version', function(value, model, oldValue){
		this.controls.dbVersion.textContent = value;

		if(oldValue == value){
			this.controls.updateWarning.style.display = '';
			clearTimeout(_updateTimer);
			_updateTimer = setTimeout(function(){
				this.controls.updateWarning.style.display = 'none';
			}.bind(this), 30000);
		}else{
			this.controls.updateWarning.style.display = 'none';
		}
	}.bind(this));
	Settings.on('change:httpd', function(value){
		self.port.emit('save', value);
	});
	Settings.on('change:categories', function(categories, model){
		self.port.emit('updateSettings', {
			categories: categories
		});
	});
	self.port.on('hostExist', function(isExist){
		if(isExist){
			this.controls.form.classList.remove('__invalid');
		}else{
			this.controls.form.classList.add('__invalid');
		}
	}.bind(this));

	this.on('updateBtn', 'click', function(){
		if(!_btnLock){
			Settings.updateDB();
		}
	});
	this.on('form', 'submit', function(e){
		e.preventDefault();
		var host = this.controls.host.value;

		Settings.change('httpd', host);
	}.bind(this));
	//-----------------------
	this.on('blockCat', 'change', function(e){
		var 	code = parseInt(e.target.value),
				categories = Settings.get('categories'),
				checked = e.target.checked;

		if(code){
			if(checked){ // add
				categories &= ~code;
			}else{ // remove
				categories |= code;
			}
			Settings.change('categories', categories);
		}
	});

	var 	categories = Settings.get('categories'),
			i = 16,
			da = document.all,
			checkbox;

	while(i <= 131072){
		checkbox = da['c' + i];

		if(!(categories & i) && checkbox){
			checkbox.checked = true;
		}
		i *= 2;
	}

	//-----------------------
	// this.on('host', 'focus', function(e){
	// 	this.controls.form.classList.remove('__invalid');
	// }.bind(this));

	this.controls.host.value = Settings.get('httpd') || '';	


	this.controls.scanTimeInput.textContent = Settings.get('scan_timeout_ms') || '';
	$4.setDataValue(this.controls.scanTimeInput, 'postfix', i18n.options_sec);
	if(Settings.get('heuristic_analysis')){
		this.controls.heuristicCheckbox.checked = true;
	}
	this.controls.compressionInput.textContent = Settings.get('max_compression_ratio') || '';
	this.controls.packerInput.textContent = Settings.get('packer_max_level') || '';
	this.controls.archiveInput.textContent = Settings.get('archive_max_level') || '';
	this.controls.mailInput.textContent = Settings.get('mail_max_level') || '';
	this.controls.containerInput.textContent = Settings.get('container_max_level') || '';

	this.on('advancedSettings', 'click', function(e){
		var advDiv = this.controls.advancedDiv;
		if(advDiv.style.display == '') {
			advDiv.style.display = 'none';
			e.target.innerText = i18n.options_show_advanced_settings;
		} else {
			advDiv.style.display = '';
			e.target.innerText = i18n.options_hide_advanced_settings;
		}
	}.bind(this));

	this.on('applyBtn', 'click', function(){
		// if(this.controls.compressionInput.textContent != '' && (this.controls.compressionInput.textContent < this.controls.compressionInput.attributes['data-empty'].value)) {
		//  	this.controls.compressionInput.parentNode.classList.add('__invalid');
		// }else{
		//  	this.controls.compressionInput.parentNode.classList.remove('__invalid');
		// 	var conf = {
		// 		scan_timeout_ms: (this.controls.scanTimeInput.textContent || this.controls.scanTimeInput.attributes['data-empty'].value) - 0,
		// 		heuristic_analysis: this.controls.heuristicCheckbox.checked,
		// 		packer_max_level: (this.controls.packerInput.textContent || this.controls.packerInput.attributes['data-empty'].value) - 0,
		// 		archive_max_level: (this.controls.archiveInput.textContent || this.controls.archiveInput.attributes['data-empty'].value) - 0,
		// 		container_max_level: (this.controls.containerInput.textContent || this.controls.containerInput.attributes['data-empty'].value) - 0,
		// 		max_compression_ratio: (this.controls.compressionInput.textContent || this.controls.compressionInput.attributes['data-empty'].value) - 0,
		// 		mail_max_level: (this.controls.mailInput.textContent || this.controls.mailInput.attributes['data-empty'].value) - 0,
		// 	}
		// 	Settings.updateFileCheckSettings(conf);
		// }
		var conf = {
			heuristic_analysis: this.controls.heuristicCheckbox.checked,
		};

		if(!(conf.scan_timeout_ms = getNum(this.controls.scanTimeInput.textContent).eqmo(0).val)){
			conf.scan_timeout_ms = this.controls.scanTimeInput.dataset.empty - 0;
			this.controls.scanTimeInput.textContent = conf.scan_timeout_ms;
		}
		if(!(conf.packer_max_level = getNum(this.controls.packerInput.textContent).eqmo(0).val)){
			conf.packer_max_level = this.controls.packerInput.dataset.empty - 0;
			this.controls.packerInput.textContent = conf.packer_max_level;
		}
		if(!(conf.archive_max_level = getNum(this.controls.archiveInput.textContent).eqmo(0).val)){
			conf.archive_max_level = this.controls.archiveInput.dataset.empty - 0;
			this.controls.archiveInput.textContent = conf.archive_max_level;
		}
		if(!(conf.container_max_level = getNum(this.controls.containerInput.textContent).eqmo(0).val)){
			conf.container_max_level = this.controls.containerInput.dataset.empty - 0;
			this.controls.containerInput.textContent = conf.container_max_level;
		}
		if(!(conf.max_compression_ratio = getNum(this.controls.compressionInput.textContent).eqmo(2).val)){
			conf.max_compression_ratio = this.controls.compressionInput.dataset.empty - 0;
			this.controls.compressionInput.textContent = conf.max_compression_ratio;
		}
		if(!(conf.mail_max_level = getNum(this.controls.mailInput.textContent).eqmo(0).val)){
			conf.mail_max_level = this.controls.mailInput.dataset.empty - 0;
			this.controls.mailInput.textContent = conf.mail_max_level;
		}

		Settings.updateFileCheckSettings(conf);


	}.bind(this));
	this.on('checkSettingsDiv', 'input', function(e){
		if(!/^\d+$/.test(e.target.textContent)){
			e.target.textContent = e.target.textContent.replace(/\D+/ig, '');
			setEndOfContenteditable(e.target);
		}
	 });
	this.controls.checkSettingsDiv.addEventListener('focus', function(e){
		if(e.target.classList.contains('visual-input')){
			e.target.parentNode.classList.add('__focus');	
		}
	}, true);
	this.controls.checkSettingsDiv.addEventListener('blur', function(e){
		if(e.target.classList.contains('visual-input')){
			e.target.parentNode.classList.remove('__focus');	
		}
	}, true);
};

self.port.on('getSettings', function(data){
	Settings.attr = data.settings;
	AdsModel.attr = data.adsData;
	FlashModel.attr = data.flashData;
	i18n = data.i18n;
	
	var optionView = new OptionView({
		el: document.body
	});
	optionView.controls.form.setAttribute('data-warning', data.warningMessage);
});
self.port.emit('getSettings');
self.port.on('updateDB', function(data){
	Settings.change('updateTimeStamp', data.updateTimeStamp);
	Settings.change('version', data.version);
});
//=======================================
// Fragment
//=======================================
{
	function Fragment(template){
		this.root = document.createDocumentFragment();
		this.alias = {};

		var 	list = template.split(/\s*(\{[^\}]*\}|\>|\,|\<+|\|)\s*/g).filter(function(s){return s && s;}),
				buf,
				i, j, len;

		len = this.isOperator(list[0]) ? list.unshift('div') : list.length;
		buf = this.parseNode(list[0].trim());
		this.root.appendChild(buf);
		this.current = (buf instanceof Text) ? this.root : buf;

		for(i = 1; i < len; i += 2){
			buf = this.parseNode(list[i+1].trim());	

			if(list[i] == '>'){
				if(this.current instanceof Text){
					this.current.parentNode.appendChild(buf)
				}else{
					this.current.appendChild(buf);
				}
				this.current = buf;
			}else if(list[i][0] == '<'){
				j = list[i].length + 1;
				while(j-- > 0 && this.current.parentNode){
					this.current = this.current.parentNode;
				}
				this.current.appendChild(buf);
				this.current = buf;
			}else if(list[i] == ','){
				this.current.parentNode.appendChild(buf);
				this.current = buf;
			}else if(list[i] == '|'){
				this.root.appendChild(buf);
				this.current = buf;
			}
		}
	}
	Fragment.prototype.getNode = function(){
		return this.root;
	}
	Fragment.prototype.isOperator = function(str){
		return str == '>' || str[0] == '<' || str == ',' || str == '|';
	}
	Fragment.prototype.parseNode = function(str){
		if(str[0] == '{'){
			var segment = str.substring(1, str.length - 1);
			if(segment[0] == '$'){
				segment = segment.substr(1);
				return this.alias[segment] = document.createTextNode('');
			}else{
				return document.createTextNode(segment);	
			}
		}else{
			var 	list = str.split(/\s*([\w\-_]+\=\"[^"]+\"|\.|\$|\@)\s*/g).filter(function(s){return s && s;}),
					i = 0,
					tag, buf,
					len = list.length;

			if(list[0] == '.' || list[0] == '$' || list[0] == '@'){
				tag = 'div';
			}else{
				tag = list[0];
				i++;
			}
			var $node = document.createElement(tag);

			for(; i < len; i += 2){
				if(list[i] == '.'){
					$node.classList.add(list[i+1]);
				}else if(list[i] == '$'){
					this.alias[list[i+1]] = $node;
				}else if(list[i] == '@'){
					buf = list[i+1].split('=');
					$node.setAttribute(buf[0].trim(), this.escapeAttrVal(buf[1]));
				}
			}
			return $node;	
		}
	}
	Fragment.prototype.escapeAttrVal = function(str){
		var val = str && str.trim();
		
		if(val){
			val = val.replace(/\"/g, '');
		}
		return val;
	}
}
function setEndOfContenteditable(contentEditableElement) {
    var range,selection;
    range = document.createRange();//Create a range (a range is a like the selection but invisible)
    range.selectNodeContents(contentEditableElement);//Select the entire contents of the element with the range
    range.collapse(false);//collapse the range to the end point. false means collapse to end rather than the start
    selection = window.getSelection();//get the selection object (allows you to change selection)
    selection.removeAllRanges();//remove any selections already made
    selection.addRange(range);//make the range you have just created the visible selection
}
// getNum("333").less(2).val || "fallback"
// @param {String} value
function getNum(value){
	if(!(this instanceof arguments.callee)){
		return new arguments.callee(value)
	}else{
		this.val = value - 0;
	}
}
getNum.prototype.less = function(num){
	if(this.val >= num){
		this.val = undefined;
	}
	return this;
}
// current value more then num
getNum.prototype.more = function(num){
	if(this.val <= num){
		this.val = undefined;
	}
	return this;
}
// equal and more
getNum.prototype.eqmo = function(num){
	if(this.val < num){
		this.val = undefined;
	}
	return this;
}


