// DOM manipulation library
var $4 = {
	// @param {string} text
	// @return {TextNode}
	t: function(text){
		return document.createTextNode(text);
	},
	// Create new HTML Element with options
	// @param {_} - First is Tag name <) console.dir($4.cr("option","value","email","textContent","Hello")); 
	// @return {HTMLElement}
	cr: function(){
		var EL = document.createElement(arguments[0]);
		for(var i = 1, m = arguments.length; i < m; i += 2){
			arguments[i] && arguments[i + 1] && (EL[arguments[i]] = arguments[i + 1]);
		}
		return EL;
	},

	// slector api
	// @param {Node} node
	// @param {String} selector -ccss selector
	// @param {Bool} getAll - get all results or first related
	// @return {NodeList|Node}
	select: function(selector, getAll, node){
		return (node || document)[getAll ? "querySelectorAll" : "querySelector"](selector);
	},
	emptyNode: function(node){
		var childNodes = node.childNodes;
		for(i = childNodes.length; i--;){
			node.removeChild(childNodes[i]);
		}
	},
	removeNode: function(node){
		node && node.parentNode && node.parentNode.removeChild(node);
	},
	removeNodes: function(nodeList){
		var len = nodeList.length;
		
		while(len--){
			this.removeNode(nodeList[len]);
		}
	},
	// Prepend node
	// @param {Node} parent
	// @param {Node} node
	prepend: function(parent, node){
		if(parent.firstChild){
			parent.insertBefore(node, parent.firstChild);
		}else{
			parent.appendChild(node);
		}
	},
	parentByTag: function(node, tagName){
		tagName = tagName.toUpperCase();
		var currentNode = node;
		
		while(currentNode.tagName != tagName && currentNode != document.body){
			currentNode = currentNode.parentNode;
		}
		
		return currentNode;
	},
	findLinkByHref: function(href){
		var 	$links = document.links;
		
		for(var i = 0; i < $links.length; i++){
			if($links[i].href == href){
				return $links[i];
			}
		}
	},
	css: function($n, o){
		for(var key in o) if(o.hasOwnProperty(key)){$n.style[key] = o[key];}
		return $n;
	},
	getStyle: function(elem, name){ 
	    if (elem.style && elem.style[name]) {
			return elem.style[name]; 
		}else if (document.defaultView && document.defaultView.getComputedStyle) { // Или методом W3C, если он существует 
			// 'textAlign' -> 'text-align' 
			name = name.replace(/([A-Z])/g, "-$1").toLowerCase(); 
			var s = document.defaultView.getComputedStyle(elem, ""); 
			return s && s.getPropertyValue(name); 
		}else if (elem.currentStyle && elem.currentStyle[name]){ // IE fix
			return elem.currentStyle[name]; 
	    }else{ 
	       return null; 
		}
	}, 
	setStyle: function(node, object){
		for(var name in object){
			if(object.hasOwnProperty(name)){
				node.style[name] = object[name];
			}	
		}
		return node;
	},
	fireEvent: function(node, eventName){
		return node.dispatchEvent(new MouseEvent(eventName, {
			'view': window,
			'bubbles': true,
			'cancelable': true
		}));
	},
};

{
	// Implementation of Event Emitter
	function Events(){
		this._handlers = Object.create(null);
	};
	// @memberOf Events - execute event callbacks
	// @param {Object} options - event options
	// @return {Bool} - if true - stop event propagation
	Events.prototype.trigger = function(name, arg1, arg2, arg3){
		var 	handlers = this._handlers[name],
				i;

		if(Array.isArray(handlers)){
			i = handlers.length;
			while(i-- > 0){
				if(handlers[i](arg1, arg2, arg3)){
					return true;
				}	
			}
		}
		return false;
	};
	// @memberOf {Events} - remove all event listeners
	Events.prototype.destroy = function(){
		for(var key in this._handlers){
			this.off(key);
		}
	};
	// @memberOf {Events} - attach callback on change
	// @param {String} name - property of model
	// @param {Function} cb - callback
	Events.prototype.on = function(name, cb){
		if(!Array.isArray(this._handlers[name])){
			this._handlers[name] = [];
		}
		this._handlers[name].push(cb);
	};
	// @memberOf {Events} - deattach event
	// @param {String} name - property of model
	// @param {Function} cb - callback
	Events.prototype.off = function(name, cb){
		var handlers = this._handlers[name];
		
		if(Array.isArray(handlers)){
			if(cb){
				var pos = handlers.indexOf(cb);
				pos != -1 && handlers.splice(pos, 1);

				if(handlers.length == 0){
					delete this._handlers[name];
				}
			}else{
				handlers.length = 0;
				delete this._handlers[name];
			}
		}
	};
	// @memberOf {Events} - attach callback on change
	// @param {String} name - property of model
	// @param {Function} cb - callback
	// @return {Function} handler
	Events.prototype.once = function(name, cb){
		if(!Array.isArray(this._handlers[name])){
			this._handlers[name] = [];
		}
		var _cb = function(args){
			this.off(name, _cb);
			return cb(args);
		}.bind(this);
		this._handlers[name].push(_cb);
		return _cb;
	};
}
//========================================
// ToolBox
//========================================
{
	function ToolBox(handlers){
		this.atTop = true;
		this.el = document.createElement('iframe');
		this._handlers = handlers;
		$4.setStyle(this.el, {
			zIndex: 4999999999,
			position: 'fixed',
			width: '100%',
			left: '0',
			top: '0',
			height: '37px',
			border: 'none',
		});
		this.controls = {
			label: $4.cr('span', 'textContent', self.options.liter.panel_description),
			saveBtn: $4.cr('button', 'className', 'base-btn drweb_btn __blue', 'textContent', self.options.liter.save),
			cancelBtn: $4.cr('button', 'className', 'base-btn drweb_btn', 'textContent', self.options.liter.cancel),
			replaceBtn: $4.cr('button', 'className', 'base-btn drweb_fit-btn'),
			restoreBtn: $4.cr('button', 'className', 'base-btn drweb_tool-icon drweb_tool-icon_show', 'title', self.options.liter.restore),
			removeBtn: $4.cr('button', 'className', 'base-btn drweb_tool-icon drweb_tool-icon_remove', 'title', self.options.liter.remove),
		};

		this.el.onload = function(e){
			var 	$body = e.target.contentDocument.body,
					$head = e.target.contentDocument.head,
					pos, eventName, component;

			$4.setStyle($body, {
				fontFamily: 'Arial',
				fontSize: '14px',
				lineHeight: '1',
				// color: '#7d7d7d',
				// background: '#fefcde',
				color: '#fff',
				background: 'rgba(0,0,0,0.7)',
				height: '36px',
				// height: '35px',
				// borderTop: '1px solid #d5d3bb',
				// borderBottom: '1px solid #d5d3bb',
				borderBottom: '1px solid #a1a1a1',
				padding: '0 0 0 24px',
				margin: '0',
				overflow: 'hidden',
			});
			$head.appendChild($4.cr('link', 'rel', 'stylesheet', 'type', 'text/css', 'href', self.options.toolbox_css));
			$body.appendChild(this.controls.replaceBtn);
			$body.appendChild(this.controls.removeBtn);
			$body.appendChild(this.controls.restoreBtn);

			$body.appendChild(this.controls.label);
			$body.appendChild(this.controls.saveBtn);
			$body.appendChild(this.controls.cancelBtn);

			this.controls.replaceBtn.onclick = function(e){
				if(!this.atTop){
					$body.classList.remove('__bottom');
					this.el.style.bottom = 'auto';
					this.el.style.top = '0';
					document.body.style.marginTop = '37px';
				}else{
					$body.classList.add('__bottom');
					this.el.style.bottom = '0';
					this.el.style.top = 'auto';
					document.body.style.marginTop = '';
				}
				this.atTop = !this.atTop;
			}.bind(this);

			for(var key in this._handlers){
				pos = key.indexOf(' ');
				if(pos != -1){
					eventName = key.substr(0, pos);
					component = key.substring(pos + 1);

					if(this.controls[component]){
						this.controls[component][eventName] = this._handlers[key].bind(this);
					}
				}
			}
		}.bind(this);
	}
	ToolBox.prototype.open = function(){
		this._cachePosition = document.body.style.position;
		document.body.appendChild(this.el);
		document.body.style.marginTop = '37px';
		document.body.style.position = 'static'; // if `relative` need to fix canvas position!
	}
	ToolBox.prototype.remove = function(){
		document.body.removeChild(this.el);
		document.body.style.marginTop = '';
		document.body.style.position = this._cachePosition; // if `relative` need to fix canvas position!
	}
	ToolBox.prototype.destroy = function(){
		for(var key in this._handlers){
			pos = key.indexOf(' ');
			if(pos != -1){
				eventName = key.substr(0, pos);
				component = key.substring(pos + 1);

				if(this.controls[component]){
					this.controls[component][eventName] = null;
				}
			}
		}
		this._handlers = {};
	};
}


var 	_HOST = window.location.host.replace('www.', ''), // aka baseURL
		_HREF = window.location.href, // aka actualURL
		_BLOCKED_ATTR = 'pf-blocked' + ~~(Math.random() * 1000),
		_dispatcher = new Events();		
//========================================
// RestoreApp2 && Selector2
//========================================
var RestoreApp2 = (function(){
	var 	_CLASS = 'rstr-mchn',
			_SVG_NS = 'http://www.w3.org/2000/svg';
			
	return {
		// @memberOf $m
		each: function(collection, callback){
			if(typeof(collection.length) != 'undefined'){
				for(var i = 0, len = collection.length; i < len; i++){
					callback(collection[i], i);
				}
			}else{
				for(var p in collection){
					if(collection.hasOwnProperty(p)){
						callback(collection[p], p);
					}
				}
			}
		},
		_disabledPool: [],
		// handlers: [],
		// count: 0,
		init: function(){
			this.destroy();
			this.svgRoot = document.createElementNS(_SVG_NS, 'svg');
			this.svgRoot.classList.add('slct-mchn_restore-canvas');
			Selector2.pickerRoot.appendChild(this.svgRoot);
			
			// this.svgRoot.onclick = function(e){
			// 	var index = e.target && e.target.getAttribute('data-id');
			// 	if(index && this.handlers[index]){
			// 		this.handlers[index]();
			// 		this.count--;
			// 	}
			// 	e.stopPropagation();
			// }.bind(this);
			this.open();
		},
		open: function(){
			var 	svgWidth = document.documentElement.scrollWidth,
					svgHeight = document.documentElement.scrollHeight,
					disabledNodes = $4.select('[link-checker=disabled]', true);
			
			this.svgRoot.setAttribute('x', 0);
			this.svgRoot.setAttribute('y', 0);
			this.svgRoot.setAttribute('width', svgWidth);
			this.svgRoot.setAttribute('height', svgHeight);
			this.svgRoot.setAttribute("viewBox", '0 0 ' + svgWidth + ' ' + svgHeight);
			
			//disabledNodes
			/*var svgIsland;
			for(var i = 0, len = disabledNodes.length; i < len; i++){
				svgIsland = this.createIland(disabledNodes[i], function(node, island){
					return function(){
						node.removeAttribute('link-checker');
						$4.removeNode(island);
						var pos = Selector2.disabledNodes.indexOf(node);

						if(pos != -1){
							Selector2.disabledNodes.splice(pos, 1);
							Selector2._onDisabledChanges();	
						}

						chrome.runtime.sendMessage({
							action: 'clearBlockRule',
							host: _HOST,
                            domString: Selector2.createFilterFromNode(node)
						});
					};
				});
				this.svgRoot.appendChild(svgIsland);
			}*/

			this.each(disabledNodes, function(node){
				var svgIsland = this.createIland(node);
				svgIsland.onclick = function(e){
					node.removeAttribute('link-checker');
					$4.removeNode(e.target);
					var pos = Selector2.disabledNodes.indexOf(node);

					if(pos != -1){
						Selector2.disabledNodes.splice(pos, 1);
						Selector2._onDisabledChanges();	
					}
				};
				this.svgRoot.appendChild(svgIsland);
			}.bind(this));

			if(Array.isArray(this.custom)){
				var 	i = this.custom.length,
						j, nodes;

				while(i-- > 0){
					nodes = document.querySelectorAll(this.custom[i]);
					j = nodes.length;

					while(j-- > 0){
						!nodes[j].getAttribute(_BLOCKED_ATTR) && this.enableHidden(nodes[j], this.custom[i]);
					}
				}
			}
		},
		enableHidden: function($node, selector){
			$node.setAttribute(_BLOCKED_ATTR, true);
			Selector2.disabledNodes.push($node);
			Selector2._onDisabledChanges();	
			var svgIsland = this.createIland($node);

			svgIsland.onclick = function(e){
				$node.setAttribute('link-checker', 'enabled');
				$4.removeNode(e.target);
				var pos = Selector2.disabledNodes.indexOf($node);

				if(pos != -1){
					Selector2.disabledNodes.splice(pos, 1);
					Selector2._onDisabledChanges();	
				}

				pos = this._disabledPool.indexOf($node);

				if(pos != -1){
					this._disabledPool.splice(pos, 1);
				}

				// chrome.runtime.sendMessage({
				// 	action: 'clearBlocks',
				// 	host: _HOST,
				// 	selector: selector,
				// });
				self.port.emit('clearBlocks', {
					host: _HOST,
					selector: selector,
				});
			}.bind(this);
			this.svgRoot.appendChild(svgIsland);
			this._disabledPool.push($node);
		},
		createIland: function(node, createHandler){
			var 	island = document.createElementNS(_SVG_NS, 'path'),
					offx = window.pageXOffset,
					offy = window.pageYOffset,
					ow = this.svgRoot.getAttribute('width'),
					islandStr,
					r = node.getBoundingClientRect();
			
			island.classList.add('slct-machn_islands');
			island.classList.add('slct-machn_red');
			islandStr = 'M' + (r.left + offx) + ' ' + (r.top + offy) + 'h' + r.width + 'v' + r.height + 'h-' + r.width + 'z';
			island.setAttribute('d', islandStr);

			return island;
		},
		destroy: function(){
			// if(this.svgRoot && this.svgRoot.parentNode){
			// 	$4.removeNode(this.svgRoot);
			// }
			// this.handlers.length = 0;
			// this.count = 0;
			if(this.svgRoot && this.svgRoot.parentNode){
				$4.removeNode(this.svgRoot);
			}
			var i = this._disabledPool.length;

			while(i-- > 0){
				this._disabledPool[i].removeAttribute('link-checker');
				this._disabledPool[i].removeAttribute(_BLOCKED_ATTR);
			}

			this._disabledPool.length = 0;
		},
	};
}());
var Selector2 = (function(){
	var 	_CLASS = 'slct-mchn',
			_SVG_NS = 'http://www.w3.org/2000/svg';

	return {
		pickerRoot: null,
		svgRoot: null,
		svgOcean: null,
		svgIslands: null,
		targetElements: undefined,
		disabledNodes: [],
		pickerPaused: function(){
			return this.pickerRoot.classList.contains('paused')
		},
		pausePicker: function(){
			this.pickerRoot.classList.add('paused');
		},
		// @param {HtmlElement} node
		createFilterFromNode: function(node, baseSelector){
			if(!node.parentNode || node.parentNode == document){
				console.warn('Can`t find unique selector `%s`', baseSelector);
				return baseSelector;
			}else{
				var 	className = node.className.trim(),
						curSelector = baseSelector ? ('>' + baseSelector) : '',
						$nodes;

				// Attention: getElementById can work with unvalid identifiers values and '\w' is 0..9 and a..z
				// So...
				if(node.id && /^[A-Za-z][\w\_\-]*$/.test(node.id)){ // 
					curSelector = '#' + node.id + curSelector;
				}else if(className){
					curSelector = '.' + className.replace(/\s+/g, '.') + curSelector;
				}else{
					curSelector = node.tagName.toLowerCase() + curSelector;
				}
				$nodes = document.querySelectorAll(curSelector);

				if($nodes && $nodes.length > 1){
					return this.createFilterFromNode(node.parentNode, curSelector);
				}
			}
			return curSelector;
		},
		unpausePicker: function(){
			this.pickerRoot.classList.remove('paused');
		},
		// @param {HtmlElement} elems
		highlightElements: function(elems, force) {
			// To make mouse move handler more efficient
			if(!force && (!elems || elems == this.targetElements)){
				return;
			}
			this.targetElements = elems;
			var offx = window.pageXOffset;
			var offy = window.pageYOffset;
			var ow = this.svgRoot.getAttribute('width');
			var ocean = [
				'M0 0',
				'h', ow,
				'v', this.svgRoot.getAttribute('height'),
				'h-', ow,
				'z'
			];
			var islands = [];
			var r = elems.getBoundingClientRect();
			ocean.push(
				'M', r.left + offx, ' ', r.top + offy,
				'h', r.width,
				'v', r.height,
				'h-', r.width,
				'z'
			);
			islands.push(
				'M', r.left + offx, ' ', r.top + offy,
				'h', r.width,
				'v', r.height,
				'h-', r.width,
				'z'
			);
			

			this.svgOcean.setAttribute('d', ocean.join(''));
			var islandPath = islands.join('');
			//console.log('Path d: `%s`', islandPath);
			islandPath && this.svgIslands.setAttribute('d', islandPath);
		},
		removeElements: function(elems) {
			var 	i = elems.length;
			
			while(i--){
				$4.removeNode(elems[i]);
			}
		},
		_lastDocWidth: 0,
		_lastDocHeight: 0,
		checkDocumentResize: function(svg){
			var 	svgWidth = document.documentElement.scrollWidth,
					svgHeight = document.documentElement.scrollHeight;
			
			if(this._lastDocWidth != svgWidth || this._lastDocHeight != svgHeight){
				this.resizeCanvas(svg, svgWidth, svgHeight);
				this._lastDocWidth = svgWidth;
				this._lastDocHeight = svgHeight;
			}
		},
		resizeCanvas: function(svg, width, height){
			svg.setAttribute('x', 0);
			svg.setAttribute('y', 0);
			svg.setAttribute('width', width);
			svg.setAttribute('height', height);
			svg.setAttribute("viewBox", '0 0 ' + width + ' ' + height);
		},
		onKeyDownHandler: undefined,
		onMouseMoveHandler: undefined,
		onClickHandler: undefined,
		disableElement: function(node){
			////Attention:  Not work if set with important 'table-cell !important'
			// node.style.display = 'none';
			node.setAttribute('cache-style', node.getAttribute('style') || '');
			node.setAttribute('style', 'display: none;');
			node.setAttribute('link-checker', 'disabled');
		},
		enableElement: function(node){
			//node.style.display = '';
			var cacheStyle = node.getAttribute('cache-style')
			
			node.setAttribute('style', cacheStyle);
			node.removeAttribute('cache-style');
			node.setAttribute('link-checker', 'enabled');
		},
		// emulate event handler
		_onDisabledChanges: function(){
			// this.toolbox.setLabel(this.disabledNodes.length);
		},
		init: function(){
			var 	bodyRect = document.body.getBoundingClientRect(),
					borderLeft = parseInt($4.getStyle(document.body, 'borderLeftWidth') || 0),
					borderTop = parseInt($4.getStyle(document.body, 'borderLeftWidth') || 0);
			
			this.onMouseMoveHandler = function(ev){
				this.checkDocumentResize(this.svgRoot);
				
				if(this.pickerPaused()){
					return;
				}

				this.svgRoot.style.pointerEvents = 'none';
				var elem = document.elementFromPoint(ev.clientX /*- leftCompensation*/, ev.clientY /*- topCompensation*/);
				
				if(elem == document.body || elem === document.documentElement){
					elem = null;
				}
				this.highlightElements(elem);
				this.svgRoot.style.pointerEvents = 'auto';
			}.bind(this);
			this.onClickHandler = function(e){
				e.stopPropagation();

				if(this.pickerPaused()){
					return;
				}
				this.disabledNodes.push(this.targetElements);
				this.disableElement(this.targetElements);
				this._onDisabledChanges();
			}.bind(this);
			this.onKeyDownHandler = function(e){
				if(e.keyCode === 27){
					var i = this.disabledNodes.length;
					while(i-- > 0){
						this.enableElement(this.disabledNodes[i]);
					}
					e.stopPropagation();
					e.preventDefault();
					this.destroy();
				}
			}.bind(this);
		},
		destroy: function(){
			RestoreApp2.destroy();
			this.disabledNodes.length = 0;
			
			if(this.pickerRoot){
				this.toolbox.destroy();
				document.removeEventListener('keydown', this.onKeyDownHandler);
				this.svgRoot.removeEventListener('mousemove', this.onMouseMoveHandler);
				this.svgRoot.removeEventListener('click', this.onClickHandler);
				document.body.removeChild(this.pickerRoot);
				this.pickerRoot = this.divDialog = this.svgRoot = this.svgOcean = this.svgIslands = this.taCandidate = null;
			}
			this.targetElements = undefined;
			this._lastDocWidth = 0;
			this._lastDocHeight = 0;
			document.body.style.marginTop = '';
			document.body.style.position = '';
		},
		start: function(){
			var that = this;
			this.pickerRoot = document.querySelector('.' + _CLASS);
			
			if(this.pickerRoot) {
				return;
			}
			this.pickerRoot = $4.cr('div', 'className', _CLASS);
			this.svgRoot = document.createElementNS(_SVG_NS, 'svg');
			this.svgRoot.classList.add('slct-mchn_block-canvas');
			
			this.svgOcean = document.createElementNS(_SVG_NS, 'path');
			this.svgIslands = document.createElementNS(_SVG_NS, 'path');
			this.svgOcean.classList.add('slct-machn_ocean');
			this.svgIslands.classList.add('slct-machn_islands');
			this.svgRoot.appendChild(this.svgOcean);
			this.svgRoot.appendChild(this.svgIslands);
			this.pickerRoot.appendChild(this.svgRoot);

			this.toolbox = new ToolBox({
				'onclick saveBtn': function(){
					var 	selectorCollection = [],
							i = that.disabledNodes.length;

					while(i-- > 0){
						selectorCollection.push(that.createFilterFromNode(that.disabledNodes[i]));
					}
					that.destroy();
				
					self.port.emit('saveBlocks', {
						host: _HOST,
					 	blocks: selectorCollection,
					});
				},
				'onclick cancelBtn': function(){
					var i = that.disabledNodes.length;
					while(i-- > 0){
						that.enableElement(that.disabledNodes[i]);
					}
					that.destroy();
				},
				'onclick restoreBtn': function(e){
					var disabledNodes = document.querySelectorAll('[link-checker=disabled]');
				
					if(that.pickerPaused()){
						this.controls.label.textContent = self.options.liter.panel_description;
						this.controls.saveBtn.style.display = '';
						this.controls.cancelBtn.textContent = self.options.liter.cancel;
						e.target.classList.remove('drweb_disabled');
						e.target.classList.add('drweb_lamp_animation');
						that.unpausePicker();
						
						for(var i = 0, len = disabledNodes.length; i < len; i++){
							disabledNodes[i].style.display = 'none';
						}
						that.svgRoot.style.display = '';
						RestoreApp2.destroy();
					}else{
						this.controls.saveBtn.style.display = 'none';
						this.controls.cancelBtn.textContent = self.options.liter.close;
						this.controls.label.textContent = self.options.liter.manualUnBlockingText;
	                    e.target.classList.add('drweb_disabled');
						e.target.classList.remove('drweb_lamp_animation');
						that.pausePicker();
						
						for(var i = 0, len = disabledNodes.length; i < len; i++){
							disabledNodes[i].style.display = '';
						}
						that.svgRoot.style.display = 'none';
						RestoreApp2.init();
					}
				},
				'onclick removeBtn': function(e){
					var $body = this.el.contentDocument.body;

					e.target.classList.add('drweb_trash_animation');
					$body.classList.add('__disabled');

					var onAnimationHandler = function(e){
						e.target.classList.remove('drweb_trash_animation');
						setTimeout(function(){
							$body.classList.remove('__disabled');							
						}, 500);
						this.controls.removeBtn.removeEventListener('mozanimationend', onAnimationHandler);
						this.controls.removeBtn.removeEventListener('animationend', onAnimationHandler);
					}.bind(this);
					this.controls.removeBtn.addEventListener('mozanimationend', onAnimationHandler);
					this.controls.removeBtn.addEventListener('animationend', onAnimationHandler);

					self.port.emit('clearBlocks', {
						host: _HOST,
					});
					
					setTimeout(function(){
						var i = that.disabledNodes.length;
						while(i-- > 0){
							that.enableElement(that.disabledNodes[i]);
						}
						that.destroy();
					}, 2000);
				},
			});

			this._onDisabledChanges();
			this.pickerRoot.appendChild(this.toolbox.el);
			document.body.appendChild(this.pickerRoot);
			document.body.style.marginTop = '40px';
			document.body.style.position = 'static'; // if `relative` need to fix canvas position!
			
			this.checkDocumentResize(this.svgRoot);
			this.svgRoot.addEventListener('click', this.onClickHandler);
			this.svgRoot.addEventListener('mousemove', this.onMouseMoveHandler);
			document.addEventListener('keydown', this.onKeyDownHandler);
			
			// right mouse btn click fire - mouseover, same with Alt - click event
			this.svgRoot.oncontextmenu = function(e){
				e.preventDefault();
				e.stopPropagation();
				
				if(this.targetElements){
					$4.fireEvent(this.targetElements, e.altKey ? 'click' : 'mouseover');
				}
			}.bind(this);
			this.svgRoot.onmousedown = function(e){
				e.stopPropagation();
			};
			this.svgRoot.onmouseup = function(e){
				e.stopPropagation();
			};
		}
	};
}());

// Informer for update page
var InformApp = (function(){
	return {
		show: function(){
			if(document.querySelector('.drweb_inform-panel')){
				return;
			}
			
			var $panel = $4.cr('div', 'className', 'drweb_inform-panel'); 
			$panel.style.display = 'none';
			$panel.appendChild($4.t(self.options.liter.informer_message));
			$panel.appendChild($4.cr('div', 'className', 'drweb_btn', 'textContent', self.options.liter.informer_reload_now, 'onclick', function(){
				window.location.reload();
			}));
			$panel.appendChild($4.cr('div', 'className', 'drweb_btn', 'textContent', self.options.liter.informer_reload_later, 'onclick', function(){
				$4.removeNode($panel);
			}));
			$panel.onmouseover = function(e){
				e.stopPropagation();
				e.preventDefault();
			};
			
			$4.prepend(document.body, $panel);
			$panel.style.display = '';
		}
	}
}());
var CleanApp = (function(){
	return {
		// @param {Array} blocks
		removeBlocks: function(blocks){
			var 	i = blocks.length,
					j, $nodes, src;

			while(i-- > 0){
				if(blocks[i].isBlocked){
					src = blocks[i].route.replace(/\"/g, '');
					$nodes = document.querySelectorAll('[src="' + src + '"]');
					j = $nodes.length;

					while(j-- > 0){
						$nodes[j].style.display = 'none';
					}
				}
			}
		},
		// @memberOf CleanApp  
		hideSocialNetworkAds: function(specialBlocks){
			var 	i = specialBlocks.length,
					idList,
					classList,
					j;
					
			while(i--){
				idList = specialBlocks[i].id.split('%');
				classList = specialBlocks[i].className.split('%');
				
				j = idList.length;
				while(j--){
					$4.removeNode(document.getElementById(idList[j]));
				}
				
				j = classList.length;
				while(j--){
					$4.removeNodes(document.getElementsByClassName(classList[j]));
				}
			}
		},
		// @memberOf CleanApp
		hideAds: function(patterns){
			var 	links = document.getElementsByTagName('a'),
					yatags = document.getElementsByTagName('yatag'),
					iframes = document.querySelectorAll('iframe');
			
			var 	regs = [],
					patLen = patterns.length,
					j = patLen,
					i,
					node;
					
			while(j--){
				regs.push(new RegExp(patterns[j]));
			}
			
			i = links.length;
			while(i--){
				node = links[i];
				if(node && node.href){
					for(j = 0; j < patLen; j++){
						if(regs[j].test(node.href)){
							node.style.display = 'none';
						}
					}
				}
			}
			
			i = iframes.length;
			while(i--){
				node = iframes[i];
				if(node && node.src){
					for(j = 0; j < patLen; j++){
						if(regs[j].test(node.src)){
							node.style.display = 'none';
						}
					}
				}
			}
			
			i = yatags.length;
			while(i--){
				node = yatags[i];
				if(node){
					node.style.display = 'none';
				}
			}
		}
	}
}());
var _NetWorkRegister = {
	socialHosts: ['vk.com', 'm.vk.com', '0.vk.com', 't.vk.com', 'm.vkontakte.ru', '0.vkontakte.ru', 't.vkontakte.ru', 'vkontakte.ru', 'facebook.com', 'fb.com', 'm.facebook.com', '0.facebook.com', 'instagram.com', 'plus.google.com', 'm.google.com'],
	get: function(host){
		return this.socialHosts.indexOf(host.replace('www.', '')) != -1;
	}
};

//================================================
// Get settings from background page
//================================================
_dispatcher.once('documentInteractive', function(){
	Selector2.init();
});
self.port.on('cleanBlocks', function(data){
	_dispatcher.once('documentComplete', function(){
		CleanApp.removeBlocks(data.blocks);
		
		// if(Array.isArray(data.manual) && data.manual.length > 0){
		// 	data.manual.forEach(function(selector){
		// 		$4.removeNode($4.select(selector));
		// 	});
		// }

		if(data.blockAds){
			if(_NetWorkRegister.get(_HOST)){ // detect if opened social network
				CleanApp.hideSocialNetworkAds(data.specSocialBlocks);
			}else{
				CleanApp.hideAds(data.adsPatterns);
			}
		}
	});
	_dispatcher.once('documentInteractive', function(){
		if(Array.isArray(data.manual) && data.manual.length > 0){
			// var 	$demoStyle = $4.cr('style', 'textContent', data.manual.join(',') + '{display:none !important;position:fixed;z-index:-10000;pointer-events:none;opacity:0;}');
			var 	$demoStyle = $4.cr('style', 'textContent', data.manual.map(function(s){return s + ':not([' + _BLOCKED_ATTR + '])';}).join(',') + '{display:none !important;position:fixed;z-index:-10000;pointer-events:none;opacity:0;}');
			$demoStyle.setAttribute('lc-style', true);
			document.head.appendChild($demoStyle);
			RestoreApp2.custom = data.manual;
		}
	});
	//========================================
	// Listen document state change
	//========================================
	if(document.readyState == 'complete'){
		_dispatcher.trigger('documentInteractive');
		_dispatcher.trigger('documentComplete');
	}else{
		document.addEventListener('readystatechange', function(){
			if(document.readyState == 'interactive'){
				_dispatcher.trigger('documentInteractive');
			}else if(document.readyState == 'complete'){
				_dispatcher.trigger('documentInteractive');
				_dispatcher.trigger('documentComplete');
			}
		});
	}
});
self.port.on('toggleManualPanel', function(){
	if(!Selector2.pickerRoot){
		Selector2.start();
	}else{
		Selector2.destroy();
	}
});
self.port.on('updateExtProperties', function(data){
	if(data.group == 'informAllTabs'){
		InformApp.show();
	}
});
self.port.emit('getBlocks', {});
