var		MozStorage = require("sdk/simple-storage"),
		MozTimers = require("sdk/timers"),
		BlockManager = require('./block_manager'),
		{xhr_tools:$xhr} = require('./xhr');

var 	BLOCK_HASH = {},
		SOCIAL_BLOCKS = [];

exports.blockHash = BLOCK_HASH;
exports.socialBlocks = SOCIAL_BLOCKS;

function CreateBlockItem(id, name, type, block, pattern){
	this.id = id;
	this.name = name;
	this.type = type;
	this.block = block,
	this.pattern = pattern;
	this.reg = new RegExp(pattern);
}
function getString(string){
    if(!string){
		return null;
	}
	var 	value = "",
			abs = string.split("-");
			
    for (i = 0, len = abs.length; i < len; i++) {
        if(abs[i]){
            value += String.fromCharCode(parseInt(abs[i]));
        }
    }
    return value;
}
function updateDB(onComplete){
	$xhr.get("http://ftp.drweb.com/pub/drweb/linkchecker/Bases/.version.json?" + Math.random(), null, function(data, xhr){
		var 	curVersion = data && data.version && data.version.message,
				timeUpdate;

		if(!MozStorage.storage.version || MozStorage.storage.version != curVersion){
			$xhr.get('http://ftp.drweb.com/pub/drweb/linkchecker/Bases/.db.json?' + Math.random(), null, function(dbj, xhr){
				if(dbj){
					BLOCK_HASH = reBuildHash({
						patterns: dbj.patterns.message.split('@'),
						names: dbj.names.message.split('@'),
						types: dbj.types.message.split('@'),
						socialURLs: dbj.socialURLs.message.split('@'),
						socialBlocksIds: dbj.socialBlocksIds.message.split('@'),
						socialBlocksClasses: dbj.socialBlocksClasses.message.split('@'),
						version: curVersion
					});
				}
				timeUpdate = Date.now();
				MozStorage.storage.updateTimeStamp = timeUpdate;
				MozStorage.storage.version = curVersion;
				onComplete && onComplete('updated', curVersion, timeUpdate);
			}, 'json');
		}else{
			timeUpdate = Date.now();
			MozStorage.storage.updateTimeStamp = timeUpdate;
			onComplete && onComplete('unnecessary', curVersion, timeUpdate);
		}
	}, 'json');
};
function reBuildHash(db){
	var 	i = db.socialURLs.length,
			hash = {},
			type,
			pattern;

	if(!MozStorage.storage.db){
		MozStorage.storage.version = db.version;
		MozStorage.storage.db = {
			patterns: db.patterns,
			names: db.names,
			types: db.types,
			socialURLs: db.socialURLs,
			socialBlocksIds: db.socialBlocksIds,
			socialBlocksClasses: db.socialBlocksClasses
		};
	}
	SOCIAL_BLOCKS.length = 0;

	while(i--){
		SOCIAL_BLOCKS.push({
			id: db.socialBlocksIds[i],
			baseURL: db.socialURLs[i],
			className: db.socialBlocksClasses[i]
		});
	}
	
	for(var i = 0, len = db.patterns.length; i < len; i++){
		type = getString(db.types[i]);
		
		if(!Array.isArray(hash[type])){
			hash[type] = [];
		}
		hash[type].push(new CreateBlockItem(
			i + 1, 
			getString(db.names[i]), 
			type, 
			'true', 
			getString(db.patterns[i])
		));
	};

	return hash;			
};
function updateDBIfReady(){
	var lastUpdated = MozStorage.storage.updateTimeStamp;

	if(lastUpdated ? (Date.now() - lastUpdated > 14400000) : true){ // > 4 hours
		updateDB();
	}
};
// @param {String} host
// @param {String} requestURL - full route
// @return {Object|false}
exports.isBlockedResource = function(host, requestURL, tabHost){
	var 	pref = MozStorage.storage.preference,
			blockList;

	blockList = BLOCK_HASH['Web Analytics'];
	if(pref.analytics_enabled && blockList){
		for(var i = 0, len = blockList.length; i < len; i++){
			if(blockList[i].reg.test(requestURL)){
				return blockList[i];
			}
		}
	}
	
	if(
		pref.social_enabled &&
		tabHost.indexOf("vk.com") == -1 && 
		tabHost.indexOf("vkontakte.ru") == -1 &&
		tabHost.indexOf("facebook.com") == -1 && 
		tabHost.indexOf("plus.google.com") == -1
	){
		if(blockList = BLOCK_HASH['Social Buttons']){
			for(var i = 0, len = blockList.length; i < len; i++){
				if(blockList[i].pattern != "http://vk.com/widget_comments.php" &&  blockList[i].reg.test(requestURL)){
					return blockList[i];
				}
			}
		}
	}
	
	blockList = BLOCK_HASH['Tracking'];
	if(pref.track_enabled && blockList){
		for(var i = 0, len = blockList.length; i < len; i++){
			if(blockList[i].reg.test(requestURL)){
				return blockList[i];
			}
		}
	}
	
	if(blockList = BLOCK_HASH['Ads']){
		var exceptionStatus = BlockManager.blocks.getAdsException(tabHost);

		if(exceptionStatus == 2 || exceptionStatus == 4){
			for(var i = 0, len = blockList.length; i < len; i++){
				if(blockList[i].reg.test(requestURL)){
					return blockList[i];
				}
			}
		}
	}
	
	return false;
}
exports.updateDBIfReady = updateDBIfReady;
exports.updateDB = updateDB;

// TODO test
exports.init = function(){
	BLOCK_HASH = reBuildHash(MozStorage.storage.db || require('data/db').db.data);

	updateDB(function(status, version, updateTimeStamp){
		// console.log('Update completed. Status: '+ status + ' version: ' + version + ' updateTimeStamp: ' + updateTimeStamp);
	});
};