var 	$pageMod = require("sdk/page-mod").PageMod,
		$url = require('sdk/self').data.url,
		$scanCore = require('./scan_core'),
		CheckLink = $scanCore.CheckLink,
		$tabs = require("sdk/tabs"),
		$wc = require('./wc_api'),
		$pref = require("sdk/simple-prefs").prefs;
		MozContext = require("sdk/context-menu");	

var 	_pageWorkers = {},
		_uiq = ~~(Math.random() * 10000),
		_contextTabId,
		_ = require("sdk/l10n").get;


// collect workers
$pageMod({
	include: '*',
	attachTo: ['top', 'existing'], // inject only at main_framesand existing pages
	contentScriptFile: $url('js/context_inject.js'),
	contentScriptWhen: 'start',
	onAttach: function(worker){
		var tabId = worker.tab && worker.tab.id;
		
		if(!tabId) return;
		
		_pageWorkers[tabId] = worker;
		worker.on('pageshow', function(){ _pageWorkers[tabId] = worker;});
		worker.on('pagehide', function(){ delete _pageWorkers[tabId];});
		worker.on('detach', function(){ delete _pageWorkers[tabId];});
		worker.port.on('update_host', function(newHost){
			// $pref.host = newHost;
			$pref.host = $wc.client.setHost(newHost);
		});
	},
	contentScriptOptions: {
		sprite: $url('img/checker_sprite.png'),
		spiner: $url('img/preloader.gif'),
		vocabulary: {
			'ctx_threat': _('ctx_threat'),
			'ctx_category': _('ctx_category'),
			'ctx_owner': _('ctx_owner'),
			'ctx_link': _('ctx_link'),
			'ctx_redirect': _('ctx_redirect'),
			'ctx_ip': _('ctx_ip'),
			'ctx_error': _('ctx_error'),
			'ctx_not_recommended': _('ctx_not_recommended'),
			'ctx_threats_not_found': _('ctx_threats_not_found'),
			'ctx_cant_check_link': _('ctx_cant_check_link'),
			'ctx_please_wait': _('ctx_please_wait'),
			'ctx_blocked_by_spider_gate': _('ctx_blocked_by_spider_gate')
		}
	}
});

function collectVirNames(obj, baseList){
	var list = baseList || [], i;

	if(Array.isArray(obj.virus)){
		i = obj.virus.length;

		while(i-- > 0){
			list.push(obj.virus[i]);
		} 
	}
	if(obj.item && obj.item.length > 0){
		i = obj.item.length;

		while(i-- > 0){
			collectVirNames(obj.item[i], list);
		}
	}

	return list;
}

// initialize context menu
MozContext.Item({
	image: $url('img/icn18.png'),
	label: _('ctx_label'),
	context: MozContext.SelectorContext('a[href]'),
	contentScript: 'self.on("click",function(node,data){node.classList.add("_current-link_' + _uiq + '");self.postMessage(node.href);});', // <-- minimum for working
	onMessage: function(link){
		_contextTabId = $tabs.activeTab.id || _contextTabId;

		var 	tabId = _contextTabId,
				_activeWorker = _pageWorkers[tabId],
				_taskId = Date.now();
		
		if(!_activeWorker){
			console.warn('Can`t found relative worker for tab: ' + tabId);
			return;
		}
		var linkChecker = new CheckLink(link, {
			ip: function(ip){
				_activeWorker.port.emit('ip', ip, _taskId);
			},
			redirect: function(url){
				link != url && _activeWorker.port.emit('redirect', url, _taskId);	
			},
			gate: function(){
				_activeWorker.port.emit('gate', _taskId);
			},
			error: function(mes, code){
				_activeWorker.port.emit('error', {
					message: mes,
					code: code,
					taskId: _taskId
				});
			},
			ipCheck: function(cat, own){
				_activeWorker.port.emit('ipCheck', {
					categories: cat.map(function(key){ return $scanCore.categoryTitle[key] || key;}),
					legal_url: own,
					taskId: _taskId
				});
			},
			urlCheck: function(cat, own){
				_activeWorker.port.emit('urlCheck', {
					categories: cat.map(function(key){ return $scanCore.categoryTitle[key] || key;}),
					legal_url: own,
					taskId: _taskId
				});
			},
			fileCheck: function(report){
				var 	virName,
						virNamesCollection = collectVirNames(report);

				// console.log('FILECHECK');
				// console.log(report);

				if(virNamesCollection.length){
					virName = virNamesCollection.map(function(o){return o.name;}).join(', ');
				}
				_activeWorker.port.emit('fileCheck', virName, _taskId);
				_linkChecker = null;
			}
		});
		
		_activeWorker.port.once('destroy', function(){
			linkChecker && linkChecker.abort && linkChecker.abort();
			linkChecker = null;
		});
		_activeWorker.port.emit('initialize', {
			selector: '_current-link_' + _uiq,
			taskId: _taskId
		});
	},
});

exports.getWorker = function getWorker(tabId){
	return _pageWorkers[tabId];
};