var		MozTabs = require("sdk/tabs"),
		$data = require("sdk/self").data,
		$pageMod = require("sdk/page-mod").PageMod,
		_ = require("sdk/l10n").get,
		$baseManager = require('./base_manager'),
		$workerManager = require('./worker_manager'),
		$pref = require("sdk/simple-prefs").prefs,
		$wc = require('./wc_api'),
		$timers = require("sdk/timers"),
		MozStorage = require("sdk/simple-storage");

var		_OPTION_PAGE = $data.url('options/optionPage.html'),
	 	_optionWorker;

// @param {Array} list1
// @param {Array} list2
// @param {Function} compareFunction
// @param {String} idProp
function compareLists(list1, list2, idProp, compareFunction){
	var 	list1Hash = {},
			list2Hash = {},
			list1Len = list1.length,
			list2Len = list2.length,
			index = list1Len,
			diffList = [],
			item;
			
	while(index--){
		list1Hash[list1[index][idProp]] = list1[index];
	}
	
	index = list2Len;
	while(index--){
		item = list2[index];
		
		if(!list1Hash[item[idProp]]){//// add new
			diffList.push(item);
		}else if(!compareFunction(item, list1Hash[item[idProp]])){
			diffList.push(item);
		}
		list2Hash[item[idProp]] = item;
	}
	
	index = list1Len;
	while(index--){
		item = list1[index];
		
		if(!list2Hash[item[idProp]]){
			diffList.push(item);
		}
	}
			
	return diffList;
}


exports.reload = function(){
	if(_optionWorker){ //// refresh option page (if opened)
		_optionWorker.tab.reload();
	}
};
exports.open = function(anchor){
	if(_optionWorker){
		_optionWorker.tab.window.activate();
		_optionWorker.tab.activate();
	}else{
		MozTabs.open({
			url: _OPTION_PAGE + (anchor ? '#' + anchor : '')
		});
	}
};
exports.init = function(){
	$pageMod({
		include: _OPTION_PAGE + '*',
		contentScriptFile: [
			$data.url('options/punycode.min.js'),
			$data.url('options/backside.js'),
			$data.url('options/optionPage.js'),
		],
		//// Attention:			
		//// 'end' needed for reloading page with options. Because when it reload by api (not when f5 press) fires `getSettings` without any data
		contentScriptWhen: 'end',
		onAttach: function(optWorker){
			_optionWorker = optWorker;
			optWorker.on('detach', function(){ // эквивалент перезагрузки страницы
				_optionWorker = undefined;
			}.bind(this));
			
			optWorker.port.on('updateSettings', function(data){
				var 	isSeverityHight = (data.social_enabled && !MozStorage.storage.preference.social_enabled) || 
							(data.analytics_enabled && !MozStorage.storage.preference.analytics_enabled) || 
							(data.track_enabled && !MozStorage.storage.preference.track_enabled),
						diffHost;
				
				if(data.flash_exceptions != undefined || data.ads_exceptions != undefined){
					//// Attention: hashTable is more useful structure for exceptions
					//// At that block we compare two lists by `id` property
					var 	currentExceptions = MozStorage.storage.preference[data.flash_exceptions ? 'flash_exceptions' : 'ads_exceptions'],
							updatedExceptions = data.flash_exceptions || data.ads_exceptions;
							
					diffHost = compareLists(currentExceptions, updatedExceptions, 'id', function(a,b){
						return a.allow == b.allow;
					}).map(function(item){
						return item.baseURL;
					});
				}

				//// store new settings:
				for(var key in data){
					if(data.hasOwnProperty(key)){
						MozStorage.storage.preference[key] = data[key];
					}
				}
				
				if(isSeverityHight || data.ads_enabled != undefined || data.flash_enabled != undefined){
					$workerManager.refreshExtProperties('informAllTabs');
				}
				
				if(Array.isArray(diffHost) && diffHost.length > 0){
					$workerManager.showInformerForHost(diffHost);
				}
			});
			optWorker.port.on('updateFileCheckSettings', function(data){
				for(var key in data){
					if(data.hasOwnProperty(key)){
						MozStorage.storage.preference[key] = data[key];
					}
				}
				// update file check settings
				$wc.client.setFileCheckConfig({
					scan_timeout_ms: Number.isInteger(data.scan_timeout_ms) ? data.scan_timeout_ms : 10000,
					cure: false,
					heuristic_analysis: !!data.heuristic_analysis,
					packer_max_level: Number.isInteger(data.packer_max_level) ? data.packer_max_level : 8,
					archive_max_level: Number.isInteger(data.archive_max_level) ? data.archive_max_level : 8,
					mail_max_level: Number.isInteger(data.mail_max_level) ? data.mail_max_level : 8,
					container_max_level: Number.isInteger(data.container_max_level) ? data.container_max_level : 8,
					max_compression_ratio: Number.isInteger(data.max_compression_ratio) ? data.max_compression_ratio : 500	
				});
			});
			optWorker.port.on('updateDB', function(){
				$baseManager.updateDB(function(status, version, timestamp){
					if(version){
						optWorker.port.emit('updateDB',{
							version: version,
							updateTimeStamp: timestamp
						});	
					}
				});
			});
			optWorker.port.on('save', function(host){
				$pref.host = $wc.client.setHost(host);
				//optWorker.tab.close();
				var 	_xhr, _timer;

				_xhr = $wc.client.whoami(function(d){
					$timers.clearTimeout(_timer);
					optWorker.port.emit('hostExist', !!d);
				});
				_timer = $timers.setTimeout(function(){	
					_xhr && _xhr.abort && _xhr.abort();
				}, 5000);
			});
			// Trigger when page ready	
			optWorker.port.on('getSettings', function(){
				var 	storagePref = MozStorage.storage.preference;

				optWorker.port.emit('getSettings', {
					settings: {
						analytics_enabled: storagePref.analytics_enabled,
						social_enabled: storagePref.social_enabled,
						track_enabled: storagePref.social_enabled,
						updateTimeStamp: MozStorage.storage.updateTimeStamp,
						version: MozStorage.storage.version,
						httpd: $pref.host,
						categories: storagePref.categories,

						// TODO init this settings
						scan_timeout_ms: storagePref.scan_timeout_ms,
						heuristic_analysis: storagePref.heuristic_analysis,
						packer_max_level: storagePref.packer_max_level,
						archive_max_level: storagePref.archive_max_level,
						mail_max_level: storagePref.mail_max_level,
						container_max_level: storagePref.container_max_level,
						max_compression_ratio: storagePref.max_compression_ratio,
					},
					flashData: {
						labelText: _('popup_flash_plugins'), //// 'Flash-плагины:'
						allowText: _('popup_allow_by_default'), //// 'Разрешены по умолчанию'
						denyText: _('popup_deny_by_default'), //// 'Запрещены по умолчанию'
						btnText: _('popup_exception'), //// 'Исключения'
						exceptions: (storagePref.flash_exceptions || []),
						popupTitle: _('popup_flash_block_exceptions'), //// 'Исключения при блокировке flash-плагинов'
						popupDescription: _('popup_you_can_create_rules_to_block_flash'), //// 'Вы можете создать правила блокировки flash-плагинов для определенных веб-сайтов.'
						tAllow: _('allow'), //// 'Разрешить'
						tDeny: _('deny_flash'), //// 'Запретить'
						enabled: storagePref.flash_enabled,
						ready: _('ready'),
					},
					adsData: {
						labelText: _('popup_ads'), //// 'Реклама:'
						allowText:  _('popup_allow_by_default_2'), //// 'Разрешена по умолчанию'
						denyText: _('popup_deny_by_default_2'), //// 'Запрещена по умолчанию'
						btnText: _('popup_exception'), //// 'Исключения'
						exceptions: (storagePref.ads_exceptions || []),
						popupTitle: _('popup_ad_block_exceptions'), //// 'Исключения при блокировке рекламы'
						popupDescription: _('popup_you_can_create_rules_for_blocking_ads_on_page'), //// 'Вы можете создать правила блокировки рекламы для определенных веб-сайтов.'
						tAllow: _('allow'), //// 'Разрешить'
						tDeny: _('deny_ads'), //// 'Запретить'
						enabled: storagePref.ads_enabled,
						ready: _('ready'),
					},
					warningMessage: _('options_host_unavailable'),
					i18n: {
						options_show_advanced_settings: _('options_show_advanced_settings'),
						options_hide_advanced_settings: _('options_hide_advanced_settings'),
						options_sec: _('options_sec')	
					}
				});

				// Check httpd available
				var 	_xhr, _timer;

				_xhr = $wc.client.whoami(function(d){
					$timers.clearTimeout(_timer);
					optWorker.port.emit('hostExist', !!d);
				});
				_timer = $timers.setTimeout(function(){	
					_xhr && _xhr.abort && _xhr.abort();
				}, 5000);
			});
		},
		contentScriptOptions: {}
	});
};
