var 	MozButtons = require('sdk/ui/button/action'),
		MozPanel = require("sdk/panel"), // https://developer.mozilla.org/en-US/Add-ons/SDK/High-Level_APIs/panel
	 	MozData = require("sdk/self").data,
	 	MozTabs = require("sdk/tabs"),
	 	TabManager = require('./tab_manager'),
	 	BaseManager = require('./base_manager'),
	 	WorkerManager = require('./worker_manager'),
	 	MozStorage = require("sdk/simple-storage"),
	 	BlockManager = require('./block_manager'), // BlockManager.blocks <- BlockService
	 	$wc = require('./wc_api'),
	 	_ = require("sdk/l10n").get;

const 	POPUP_HEIGHT = 228,
		POPUP_WIDTH = 360;

var		_popupPanel,
		_actionBtn;

function changeBadge(tabId, count){
	// Original code
    /*var icon = "./img/icn18_10.png";
    
	switch(count){
		case 0: icon = "./img/icn18.png"; break;
		case 1: icon = "./img/icn18_1.png"; break;
		case 2: icon = "./img/icn18_2.png"; break;
		case 3: icon = "./img/icn18_3.png"; break;
		case 4: icon = "./img/icn18_4.png"; break;
		case 5: icon = "./img/icn18_5.png"; break;
		case 6: icon = "./img/icn18_6.png"; break;
		case 7: icon = "./img/icn18_7.png"; break;
		case 8: icon = "./img/icn18_8.png"; break;
		case 9: icon = "./img/icn18_9.png"; break;
	}*/

	var 	path = './img/icn18',
			isTurnOff = $wc.client.isLock;

	if(!count || count < 10){
		path += '_' + count;
	}else if(count > 9){
		path += '_10';
	}

	if(count >= 0 && isTurnOff){
		path += '_off';
	}

	path += '.png';

	_actionBtn.state("tab", { // for current active tab
		icon: path
	});
};
// transform settings to select value (4 popup)
function settings2selectValue(exceptionCode){
	var val;
	switch(exceptionCode){
		case 2: val = 'denyHere'; break;
		case 1: val = 'allowHere'; break;
		case 3: val = 'defaultAllow'; break; 
		case 4: val = 'defaultDeny'; break;
	}
	return val;
}
// TODO this._$optionWorker

exports.init = function(){
	_popupPanel = MozPanel.Panel({
		contentURL: MozData.url("popup/popupPanel2.html"),
		contentScriptFile: MozData.url("popup/popupPanel2.js"),
		width: POPUP_WIDTH,
		height: POPUP_HEIGHT,
		contentScriptWhen: 'start',
        contentScriptOptions: {
            option_title: _('popup_icon_options_title'),
            blocking_title: _('popup_icon_blocking_title')
        },
        onHide: function(){
        	// clean DOM before opening
        	_popupPanel.port.emit('cleanUp');
        	_popupPanel.resize(POPUP_WIDTH, POPUP_HEIGHT);
        }
	});
	_actionBtn = MozButtons.ActionButton({
		id: "linkChecker",
		label: "Link Checker",
		icon: {
			"18": "./img/icn18.png",
			"32": "./img/icn32.png",
			"36": "./img/icn36.png",
			"64": "./img/icn64.png"
		},
		onClick: function(state) {
			_popupPanel.show({
				position: _actionBtn
			});
		}
	});

	_popupPanel.on("show", function(){
		_popupPanel.port.emit("show");
		changeBadge(MozTabs.activeTab.id, TabManager.getBlockRequests(MozTabs.activeTab.id).length);
	});
	_popupPanel.port.on("text-entered", function(text){
		_popupPanel.hide();
	});	
	_popupPanel.port.on('resize', function(data){
		if(data.height){
			_popupPanel.resize(POPUP_WIDTH, data.height + 5);
		}
	});
	_popupPanel.port.on('popupReady', function(data){
		var 	tabId = MozTabs.activeTab.id,
				tabHost = TabManager.cache.getHost(tabId),
			 	adsExceptionStatus = BlockManager.blocks.getAdsException(tabHost),
				flashExceptionStatus = BlockManager.blocks.getAdsException(tabHost, true);

		_popupPanel.port.emit('refreshBlocks', {
			blocks: TabManager.cache.getBlocks(tabId),
			tabHost: tabHost, // cash value for change operation
			adsValue: settings2selectValue(adsExceptionStatus),
			flashValue: settings2selectValue(flashExceptionStatus),
			flashEnabled: MozStorage.storage.preference.flash_enabled,
			adsEnabled: MozStorage.storage.preference.ads_enabled
		});
	});
	
	// TODO move this code at settings api
	_popupPanel.port.on('updateException', function(data){
		var $settingsApi = require('./settings_api');

		$settingsApi.updateBlockStatus(data.host, data.value, data.type);
		//// A little bit of shit. Here and at Chrome version, we show informer for current host. That is not correct, because sometimes it not necessary
		WorkerManager.showInformerForHost(data.host);
		//// refresh option page (if opened)
		require('./option_dialog').reload();
	});
	
	_popupPanel.port.on('toggleManualBlock', function(){
		var		tabId = MozTabs.activeTab.id;
		
		WorkerManager.toggleManualPanel(tabId);
		_popupPanel.hide();
	});
	
	// TODO move to option_page
	_popupPanel.port.on('openSettingsPage', function(){
        _popupPanel.hide();
        require('./option_dialog').open();
	});
    
    _popupPanel.port.on('navigate', function(url){
        _popupPanel.hide();
        MozTabs.open({
            url: url
        });
    });
    
    // Function is turn off
    // _popupPanel.port.on('adsReport', function(agent){
    //     var tabURL = MozTabs.activeTab.url;
    //     if(/https?:\/\//.test(tabURL)){
    //     	 _GA.sendEvent('adsReport', 'firefox', agent + '|' + tabURL, 1);
    //     }
    // });
    // Maybe: move updateDBIfReady(); and activate listener at base_manager init
    MozTabs.on('activate', function(tab){
		BaseManager.updateDBIfReady();
		changeBadge(tab.id, TabManager.getBlockRequests(tab.id).length);
	});
	MozTabs.on('ready', function(tab){
		if(MozTabs.activeTab && MozTabs.activeTab.id == tab.id){ // If active tab
			changeBadge(tab.id, TabManager.getBlockRequests(tab.id).length);
		}
	});
};

exports.changeBadge = changeBadge;
