const 	{Cc, Cu, Ci, Cr } = require('chrome');
// Cu.import("resource://gre/modules/Services.jsm");
// Services.obs.addObserver(httpRequestObserver, "http-on-modify-request", false);
// Services.obs.removeObserver(httpRequestObserver, "http-on-modify-request");
var 	$events = require('sdk/system/events'),
		$tabUtils = require('sdk/tabs/utils'),
		$d = require('./debug.js'),
		$data = require('sdk/self').data,
		$workers = require("sdk/content/worker"),
		TabManager = require('./tab_manager'),
		BaseManager = require('./base_manager'),
		BlockManager = require('./block_manager'),
		$scan_core = require('./scan_core');

// Function to detect tab relative httpRequest
// @param {subject.QueryInterface(Ci.nsIHttpChannel)} httpChannel
// @return {Object} aTab
//		@return {} aTab.owner
//		@return {} aTab.linkedBrowser:
//		@return {Int} aTab._tPos - index at window
//		@return {Bool} aTab.closing
//		@return {Bool} aTab._fullyOpen
//		@return {Object} aTab.tmp
//		@return {Object} aTab._dragPath
//		@return {timestamp} aTab._lastAccessed
// @return {Object} contentWindow
//		@return {Int} contentWindow.tabId
function _loadContextGoodies(httpChannel, subject){
    var loadContext;
	
	if (!httpChannel.notificationCallbacks) {
		return null;
	}
	// TODO try catch maybe unused
    try {
        var interfaceRequestor = httpChannel.notificationCallbacks.QueryInterface(Ci.nsIInterfaceRequestor);
        try {
            loadContext = interfaceRequestor.getInterface(Ci.nsILoadContext);
        } catch (ex) {
            try {
                loadContext = subject.loadGroup.notificationCallbacks.getInterface(Ci.nsILoadContext);
            } catch (ex2) {
				return null;
			}
        }
    } catch (ex0) {
		return null;
	}

    if (!loadContext) {
        // no load context so don't do anything although you can run this, which is your old code this probably means that its loading an ajax call or like a google ad thing
        return null;
    }
	
	try {
		var contentWindow = loadContext.associatedWindow;
		
		if (contentWindow) {
			var 	aDOMWindow = contentWindow.top
						.QueryInterface(Ci.nsIInterfaceRequestor)
						.getInterface(Ci.nsIWebNavigation)
						.QueryInterface(Ci.nsIDocShellTreeItem)
						.rootTreeItem
						.QueryInterface(Ci.nsIInterfaceRequestor)
						.getInterface(Ci.nsIDOMWindow),
					gBrowser = aDOMWindow.gBrowser,	
					aTab = gBrowser._getTabForContentWindow(contentWindow.top); //this is the clickable tab xul element, the one found in the tab strip of the firefox window, aTab.linkedBrowser is same as browser var above //can stylize tab like aTab.style.backgroundColor = 'blue'; //can stylize the tab like aTab.style.fontColor = 'red';
					
			return {
				aDOMWindow: aDOMWindow,
				gBrowser: gBrowser,
				aTab: aTab,
				tabId: $tabUtils.getTabId(aTab), // Attention:  tabUtils.getTabURL(aTab) can return tabURL, but we can use window.location.href!
				contentWindow: contentWindow
			};
		} else {
			// this channel does not have a window, its probably loading a resource
			// this probably means that its loading an ajax call or like a google ad thing
			return null;
		}
	} catch(ex2) {
		return null;
	}
}
// Attention: Firefox may have a bug - it can't open inner extension page (only inner page!) that has `://` at search or hash parts of url adress. Founded at FF39.
function _serializeRequestUrl(url){
	var 	buf = url.split('://');
	return 'original=' + encodeURIComponent(buf[1]) + '&protocol=' + buf[0];
}

var 	HTML_MIME = /text\/html/i,
		HTTP_REG = /https?\:\/\//i,
		SWF_REG = /\.(?:swf|flv)(\?|#|$)/i,
		WWW_REG = /^www\./i;

exports.httpRequestObserver = {
	// @param {OBject} data - at this case it is empty object ({}); Not documentated
    observe: function (subject, topic, data) {
        var 	httpChannel, requestURL, goodies, acceptHeader, refererHeader, host, tabId;

        if (topic == "http-on-modify-request") {
            httpChannel = subject.QueryInterface(Ci.nsIHttpChannel);
            requestURL = httpChannel.URI.spec;
            host = httpChannel.URI.host.replace(WWW_REG, '');

            if(goodies = _loadContextGoodies(httpChannel)){
            	// httpChannel.getRequestHeader('Referer'); can produce exception if header was not founded
            	var _headers = {};

				httpChannel.visitRequestHeaders(function(header, value){
					if(header == 'Accept'){
						acceptHeader = value;
					}else if(header == 'Referer'){
						refererHeader = value;
					}
					_headers[header] = value;
				});
            	tabId = goodies.tabId;

            	if(goodies.contentWindow == goodies.contentWindow.top && HTML_MIME.test(acceptHeader)){ // is main_frame
            		if(refererHeader && SWF_REG.test(refererHeader) || SWF_REG.test(requestURL)){
            			return;
            		}

            		TabManager.cache.updateTabURL(tabId, requestURL);
           		
            		var res = $scan_core.check_host(host, requestURL, function(cat, own){
            			// $d.log('WARNING %s', requestURL);
						goodies.contentWindow.location = $data.url('page/warning_page.html?cat=' + cat + (own ? '&owner=' + own.replace(HTTP_REG, '') : '') + '&' + _serializeRequestUrl(requestURL));
						// Attention: You may create worker, but it can't communicate with page!
						// var worker = $workers.Worker({
						// 	window: goodies.contentWindow,
						// });
					}, function(){ // open original page
						goodies.contentWindow.postMessage('completed', '*');
						goodies.contentWindow.location = requestURL;
						// $d.log('RESTORE %s', requestURL);
					}, httpChannel.requestMethod);

            		// console.log('\tINIT check: ' + httpChannel.URI.host + ' RES: ' +  (res ? 'block' : 'noblock'));
					if (res) { // redirect on page with progress
						// $d.log('MAIN FRAME: %s, path: `%s` ', httpChannel.URI.host, httpChannel.URI.path);
						goodies.contentWindow.location = $data.url('page/progress_page.html?' + _serializeRequestUrl(requestURL));
						// $d.log('PROCESS %s', requestURL);
					}
					return;
            	}
            } 

            // For examle flash can be without tabId
            // We use tabHost as identifier for ads and flash block rules. So if we does not have tabId, we can't detect block rules for this tab...
            // And we use host of request. This solution is not 100% correct.

            var 	tabHost = TabManager.cache.getHost(tabId),
					blockObj;

			if(!tabHost){
				httpChannel.visitRequestHeaders(function(header, value){
					if(header == 'Referer'){
						tabHost = (value.split('/')[2] || '').replace('www.', '');
					}
				});
			}
			blockObj = BaseManager.isBlockedResource(host, requestURL, tabHost || host);

			if(typeof(blockObj) == 'object'){ // abort request
				tabId && TabManager.cache.storeBlock(tabId, {
					route: requestURL,
					host: host,
					reason: blockObj.type,
					name: blockObj.name,
					isBlocked: blockObj.block == 'true'
				});
				httpChannel.cancel(Cr.NS_BINDING_ABORTED);
			}

			if(SWF_REG.test(requestURL)){
				var exception = BlockManager.blocks.getAdsException(tabHost, true);

			    if(exception == 2 || exception == 4){
			        httpChannel.cancel(Cr.NS_BINDING_ABORTED);        
			    }
			}

            return;
        }
    }
};

