var 	$api = require('./wc_api'),
		$ad = require('./authorization_dialog'),
		 _ = require("sdk/l10n").get,
		MozStorage = require("sdk/simple-storage"),
		{xhr_tools:$xhr} = require('./xhr');

// Attention! 0x1 - for 'clean' resource!
// 0x2 - skip checking, once
// 0x4 - skip, add at white list
// 0x4, 0x8 - are reserved
var CAT_LIST = {
	UC_INFECTION_SOURCE: 0x10,
	UC_NOT_RECOMMENDED: 0x20,
	UC_ADULT_CONTENT: 0x40,
	UC_VIOLENCE: 0x80,
	UC_WEAPONS: 0x100,
	UC_GAMBLING: 0x200,
	UC_DRUGS: 0x400,
	UC_OBSCENE_LANGUAGE: 0x800,
	UC_CHATS: 0x1000,
	UC_TERRORISM: 0x2000,
	UC_FREE_EMAIL: 0x4000,
	UC_SOCIAL_NETWORKS: 0x8000,
	UC_OWNERS_NOTICE: 0x10000,
	UC_BLACK_LIST: 0x20000
},
SKIP_ONCE = 0x2,
SKIP = 0x4

var WHITE_LIST = {
	'localhost': 1,
};

var cat_core = {
	list: {},
	owner: {},
	addCat: function(host, value){
		this.list[host] |= value;
	},
	removeCat: function(host, value){
		this.list[host] &= ~value;
	},
	contains: function(host, value){
		return (this.list[host] & value);
	},
	skip: function(host, isOnce){
		this.addCat(host, !isOnce ? SKIP : SKIP_ONCE);
	},
};
function code2categories(code){
	var list = [];

	for(key in CAT_LIST){
		if(code & CAT_LIST[key]){
			list.push(key);
		}
	}

	return list;
};

exports.CAT_LIST = CAT_LIST;
exports.cat_core = cat_core; 
exports.code2categories = code2categories;
exports.categoryTitle = {
	UC_INFECTION_SOURCE: _('UC_INFECTION_SOURCE'),
	UC_NOT_RECOMMENDED: _('UC_NOT_RECOMMENDED'),
	UC_ADULT_CONTENT: _('UC_ADULT_CONTENT'),
	UC_VIOLENCE: _('UC_VIOLENCE'),
	UC_WEAPONS: _('UC_WEAPONS'),
	UC_GAMBLING: _('UC_GAMBLING'),
	UC_DRUGS: _('UC_DRUGS'),
	UC_OBSCENE_LANGUAGE: _('UC_OBSCENE_LANGUAGE'),
	UC_CHATS: _('UC_CHATS'),
	UC_TERRORISM: _('UC_TERRORISM'),
	UC_FREE_EMAIL: _('UC_FREE_EMAIL'),
	UC_SOCIAL_NETWORKS: _('UC_SOCIAL_NETWORKS'),
	UC_OWNERS_NOTICE: _('UC_OWNERS_NOTICE'),
	UC_BLACK_LIST: _('UC_BLACK_LIST')
};

function clearHostFromPort(hostString){
	var pos = hostString.indexOf(':');
	return  pos > 0 ? hostString.substr(0, pos) : hostString;
}

function Waterfall(){
	this.taskList = [];
}
Waterfall.prototype.add = function(cb){
	this.taskList.push(cb);	
};
Waterfall.prototype.prepend = function(cb){
	this.taskList.unshift(cb);	
};
Waterfall.prototype.resolve = function(obj, scope){
	var task = this.taskList.shift();
	
	if (task) {
		task(this.resolve.bind(this), obj, scope);	
	}	
};
// Attention: without emptyGet!
exports.check_host = function(host, url, blockHandler, redirectHandler, method){
	if(WHITE_LIST[clearHostFromPort(host)] || $api.client.isLock) return;

	var 	category = cat_core.list[host],
			blockCat = MozStorage.storage.preference.categories;

	if (!category && method != 'POST') {
		new CheckHost(host, url, function(cat, own){ // onResult
			cat &= ~blockCat;

			if(cat > 2){
				blockHandler(cat, own);
			}else{
				redirectHandler();	
			}
		}, function(){ // onError
			$api.client.isLock = true; 
			$api.client.monitorHost();
			redirectHandler();
		});
		return true;
	} else if (category == 1 || (category & SKIP)) { // skip
		// .... mmm, nothing?
	} else if (category & SKIP_ONCE) { // skip
		cat_core.removeCat(host, SKIP_ONCE);
	} else if (category > 1) { // use cache result
		category &= ~blockCat;
		if(category > 1) blockHandler(category, cat_core.owner[host]);
	}
	return false;
}

// ==================================================================
//	CheckHost
// ==================================================================
function CheckHost(host, url, onResult, onError){
	this.wf = new Waterfall();
	this.wf.add(this.initWorker);
	this.wf.resolve({
		host: host,
		onResult: onResult,
		onError: onError,
		url: url
	}, this);
}
CheckHost.prototype.initWorker = function(next, conf, $scope){
	$api.getIp(conf.host, function(ip, code){
		if (ip) {
			conf.ip = ip
			$scope.wf.prepend($scope.ipWorker);
		} else {
			$scope.wf.prepend($scope.urlWorker);
		}
		next && next(conf, $scope);
	});
};
CheckHost.prototype.ipWorker = function(next, conf, $scope){
	$api.client.categorizeIp(conf.ip, function(d, r){
		var 	errCode = d && d.error && d.error.code,
				host = conf.host;

		/*if (errCode == 'EC_NOT_AUTHORIZED' || errCode == 'EC_INVALID_TOKEN') {
			$ad.open(function(authSuccess){
				if (authSuccess) {
					$scope.wf.prepend($scope.ipWorker);
					next && next(conf, $scope);
				} else { // if user close popup without success autorization
					cat_core.skip(host, true);
					conf.redirectHandler();
				}
			}, $api.client);
		} else*/ if (d && !errCode) {
			if (d.category) {
				cat_core.addCat(host, CAT_LIST[d.category]);
			}
			if (Array.isArray(d.categories)) {
				for(var i = 0, cat; i < d.categories.length; i++){
					cat = d.categories[i];
					cat_core.addCat(host, CAT_LIST[cat]);
				}
			}
			if (d.legal_url) {
				cat_core.owner[host] = d.legal_url;
			}
			if (!cat_core.list[host]) { // try to check url!
				$scope.wf.prepend($scope.urlWorker);
				next && next(conf, $scope);
			} else {
				conf.onResult(cat_core.list[host], cat_core.owner[host]);
			}
		} else { // you have another troubles
			cat_core.skip(host, true);
			conf.onError();
		}
	});	
};
CheckHost.prototype.urlWorker = function(next, conf, $scope){
	$api.client.categorizeUrl(conf.url, function(d, r){
		var 	errCode = d && d.error && d.error.code,
				host = conf.host;

		/*if (errCode == 'EC_NOT_AUTHORIZED' || errCode == 'EC_INVALID_TOKEN') {
			$ad.open(function(authSuccess){
				if (authSuccess) {
					$scope.wf.prepend($scope.urlWorker);
					next && next(conf, $scope);
				} else { // if user close popup without success autorization
					cat_core.skip(host, true);
					conf.redirectHandler();
				}
			}, $api.client);
		} else*/ if (d && !errCode) {
			if (d.category) {
				cat_core.addCat(host, CAT_LIST[d.category]);
			}
			if (Array.isArray(d.categories)) {
				for(var i = 0, cat; i < d.categories.length; i++){
					cat = d.categories[i];
					cat_core.addCat(host, CAT_LIST[cat]);
				}
			}
			if (d.legal_url) {
				cat_core.owner[host] = d.legal_url;
			}
			if (!cat_core.list[host]) {
				cat_core.list[host] = 1; // Clean resource;
			}
			conf.onResult(cat_core.list[host], cat_core.owner[host]);
		} else { // you have another troubles
			cat_core.skip(host, true);
			conf.onError();
		}
	});	
};

// ==================================================================
//	CategorizeUrl
// ==================================================================
function CategorizeUrl(url, oncomplete){
	this.wf = new Waterfall();
	this.wf.add(this.emptyGetWorker);
	this.wf.add(this.categorizeWorker);
	this.wf.add(oncomplete);
	this.wf.resolve(url, this);
}
CategorizeUrl.prototype.emptyGetWorker = function(next, url, $scope){
	$xhr.emptyGet(url, function(status, finalUrl){
		next && next(finalUrl, $scope);
	});
};
CategorizeUrl.prototype.categorizeWorker = function(next, url, $scope){
	if (url) {
		$api.client.categorizeUrl(url, function(d, r){
			var errCode = d && d.error && d.error.code;
			/*if (errCode == 'EC_NOT_AUTHORIZED' || errCode == 'EC_INVALID_TOKEN') {
				$ad.open(function(authSuccess){
					if (authSuccess) {
						$scope.wf.prepend($scope.categorizeUrl);
						next && next(url, $scope);
					} else { // if user close popup without success autorization
						next && next({error: 'check_abort'});
					}
				}, $api.client);
			} else*/ if (d && !errCode) {
				var 	categories = 0,
						i = d.categories.length;
				
				while(i-- > 0){
					categories |= CAT_LIST[d.categories[i]];
				}

				categories & ~MozStorage.storage.preference.categories;

				next && next({
					category: d.category,
					categories: categories
				});
			} else { // you have another troubles
				next && next({error: 'error_abort'});
			}
		});	
	} else {  // network troubles
		next && next({error: 'error_abort'});
	}
};

// ==================================================================
//	CheckLink
// ==================================================================
// @param {String} link - link to check
// @param {Object} handlers - collection of event handlers
//		{Function} urlCheck
//		{Function} ipCheck
//		{Function} error
//		{Function} ip
//		{Function} redirect
//		{Function} fileCheck
function CheckLink(link, handlers){
	this.wf = new Waterfall();
	this.on = handlers || {};
	this.wf.add(this.emptyGetWorker);
	this.wf.add(this.detectIpWorker);
	this.wf.resolve(link, this);
	this.contentRedirectCount = 0;
}
// detect redirect
CheckLink.prototype.emptyGetWorker = function(next, url, $scope){
	$scope.curRequest = $xhr.emptyGet(url, function(status, finalUrl, statusText){
		// next && next(finalUrl, $scope);
		if(finalUrl){
			var 	host = finalUrl.split('/')[2],
					conf = {
						url: finalUrl,
						host: host
					};

			$scope.on.redirect && $scope.on.redirect(finalUrl);

			if(statusText && statusText == 'Blocked by SpIDer Gate' && $scope.on.gate){
				$scope.on.gate();
			}else{
				$api.getIp(host, function(ip, code){
					if (ip) {
						conf.ip = ip;
						$scope.wf.prepend($scope.ipWorker);
					} else {
						$scope.wf.prepend($scope.urlWorker);
					}
					$scope.on.ip && $scope.on.ip(ip);
					next && next(conf, $scope);
				});	
			}
		}else{ // troubles with network 
			$scope.on.error && $scope.on.error('Network troubles');
		}
	});
};
// DEPRICATEDL
/*CheckLink.prototype.detectIpWorker = function(next, url, $scope){
	if(url){
		var 	host = url.split('/')[2],
				conf = {
					url: url,
					host: host
				};

		$scope.on.redirect && $scope.on.redirect(url);
		$api.getIp(host, function(ip, code){
			if (ip) {
				conf.ip = ip;
				$scope.wf.prepend($scope.ipWorker);
			} else {
				$scope.wf.prepend($scope.urlWorker);
			}
			$scope.on.ip && $scope.on.ip(ip);
			next && next(conf, $scope);
		});
	}else{ // troubles with network 
		$scope.on.error && $scope.on.error('Network troubles');
	}
};*/
CheckLink.prototype.ipWorker = function(next, conf, $scope){
	$scope.curRequest = $api.client.categorizeIp(conf.ip, function(d, r){
		var 	errCode = d && d.error && d.error.code,
				host = conf.host;

		/*if (errCode == 'EC_NOT_AUTHORIZED' || errCode == 'EC_INVALID_TOKEN') {
			$ad.open(function(authSuccess){
				if (authSuccess) {
					$scope.wf.prepend($scope.ipWorker);
					next && next(conf, $scope);
				} else { // if user close popup without success autorization
					$scope.on.error && $scope.on.error('Not autorized', errCode);
				}
			}, $api.client);
		} else */if (d && !errCode) {
			if (d.category) {
				cat_core.addCat(host, CAT_LIST[d.category]);
			}
			if (Array.isArray(d.categories)) {
				for(var i = 0, cat; i < d.categories.length; cat = d.categories[i], i++){
					cat_core.addCat(host, CAT_LIST[cat]);
				}
			}
			if (d.legal_url) {
				cat_core.owner[host] = d.legal_url;
			}

			if (!cat_core.list[host]) { // try to check url!
				$scope.wf.prepend($scope.urlWorker);
			} else {
				$scope.wf.prepend($scope.fileWorker);
			}
			$scope.on.ipCheck && $scope.on.ipCheck(code2categories(cat_core.list[host]), cat_core.owner[host]);
			next && next(conf, $scope);
		} else { // you have another troubles
			$scope.on.error && $scope.on.error('Troubles with network connection', errCode);
		}
	});	
};
CheckLink.prototype.urlWorker = function(next, conf, $scope){
	$scope.curRequest = $api.client.categorizeUrl(conf.url, function(d, r){
		var 	errCode = d && d.error && d.error.code,
				host = conf.host;

		/*if (errCode == 'EC_NOT_AUTHORIZED' || errCode == 'EC_INVALID_TOKEN') {
			$ad.open(function(authSuccess){
				if (authSuccess) {
					$scope.wf.prepend($scope.urlWorker);
					next && next(conf, $scope);
				} else { // if user close popup without success autorization
					$scope.on.error && $scope.on.error('Not autorized', errCode);
				}
			}, $api.client);
		} else*/ if (d && !errCode) {
			if (d.category) {
				cat_core.addCat(host, CAT_LIST[d.category]);
			}
			if (Array.isArray(d.categories)) {
				for(var i = 0, cat; i < d.categories.length; cat = d.categories[i], i++){
					cat_core.addCat(host, CAT_LIST[cat]);
				}
			}
			if (d.legal_url) {
				cat_core.owner[host] = d.legal_url;
			}
			if (!cat_core.list[host]) {
				cat_core.list[host] = 1; // Clean resource;
			}

			$scope.on.urlCheck && $scope.on.urlCheck(code2categories(cat_core.list[host]), cat_core.owner[host]);

			$scope.wf.prepend($scope.fileWorker);
			next && next(conf, $scope);
		} else { // you have another troubles
			$scope.on.error && $scope.on.error('Troubles with network connection', errCode);
		}
	});	
};
CheckLink.prototype.RED_HEADER_PATTERN = /content="\d;\s*URL=[\'\"]?([^"']+)[\'\"]?"/i;
CheckLink.prototype.fileWorker = function(next, conf, $scope){
	if(!$scope.on.fileCheck){
		next && next(conf, $scope);
		return;
	}
	$scope.curRequest = $xhr.downloadFile(conf.url, function(data, xhr){
		if(!$scope.contentRedirectCount){ // only one redirect at depth
			$scope.contentRedirectCount++;
			var match = $scope.RED_HEADER_PATTERN.exec(data);

			if(match && match[1]){
				$scope.wf.prepend($scope.detectIpWorker);
				next && next(match[1], $scope);
				return;
			}
		}
		$scope.curRequest = $api.client.checkFile2(data, function(d, r){
			var code = d && d.error && d.error.code;

			if(code || !d || !d.scan_report){
				$scope.on.error && $scope.on.error('Troubles while check file', code);
			} else { 
				$scope.on.fileCheck && $scope.on.fileCheck(d.scan_report);
			}
			next && next(conf, $scope);
		});
	});
};
CheckLink.prototype.abort = function(){
	if(this.curRequest && this.curRequest.abort){
		this.curRequest.abort();
	}
};

exports.CategorizeUrl = CategorizeUrl;
exports.CheckHost = CheckHost;
exports.CheckLink = CheckLink;