var	 	MozTabs = require("sdk/tabs"),
	 	_ = require("sdk/l10n").get;

function TabInstance(){
	this.tabURL = '';
	this.baseURL = '';
	this.blockedRequests = [];
};
TabInstance.prototype = {
	// @param {BlockedObject} data
	addBlockResource: function(data){
		var len = this.blockedRequests.length;
		while(len--){ // prevent dublicating! 
			if(
                this.blockedRequests[len].route == data.route  ||
                (this.blockedRequests[len].name == data.name && this.blockedRequests[len].reason != 'Ads')
            ){
				return;
			}
		}

		this.blockedRequests.push(data);
	}
};

var TAB_CACHE = {
	tabs: {},
	// @memberOf TabCache // TabCache.updateTabURL(id, url)
	updateTabURL: function(tabId, tabURL){
		if(!this.tabs[tabId]){
			this.tabs[tabId] = new TabInstance();
		}
		this.tabs[tabId].tabURL = tabURL;
		this.tabs[tabId].baseURL = tabURL.split('/')[2].replace(/^www\./, '');
	},
	// TabCache.getTabURL(tabId)
	getTabURL: function(tabId){
		return this.tabs[tabId] ? this.tabs[tabId].tabURL : '';
	},
	// TabCache.getHost()
	getHost: function(tabId){
		if(this.tabs[tabId]){
			return this.tabs[tabId].baseURL;
		}else{
			var 	i = MozTabs.length,
					url;

			while(i-- > 0){
				if(MozTabs[i] && MozTabs[i].id == tabId){
					url = (MozTabs[i].url.split('/')[2] || '').replace(/^www\./, '');
					break;
				}
			}
			return url;
		}
	},
	add: function(tabId){
		if(!this.tabs[tabId]){
			this.tabs[tabId] = new TabInstance();
		}else{
			this.tabs[tabId].blockedRequests.length = 0;
		}
	},
	remove: function(tabId){
		if(this.tabs[tabId]){
			this.tabs[tabId] = null;
		}
	},
	storeBlock: function(tabId, data){
		if(!this.tabs[tabId]){
			this.tabs[tabId] = new TabInstance();
		}
		this.tabs[tabId].addBlockResource(data);
	},
	getBlocks: function(tabId){
		return this.tabs[tabId] ? this.tabs[tabId].blockedRequests : [];
	}
};

exports.cache = TAB_CACHE;
exports.init = function(){
	// 	{String} tab.id - tab id
	//	{Object} tab.window - window
	MozTabs.on('open', function(tab){
		tab.on('close', function(){ // flush cache
			TAB_CACHE.remove(tab.id);
		}.bind(this))
	});
};
exports.getBlockRequests = function(tabId){
    return TAB_CACHE.getBlocks(tabId).filter(function(item){
        return item.reason != 'Tracking' && item.reason != 'Ads';
    });
};


