const 	{Cc, Cu, Ci, Cr, components} = require('chrome');
var		DnsService = Cc["@mozilla.org/network/dns-service;1"].createInstance(Ci.nsIDNSService),
 		ThreadManager = Cc["@mozilla.org/thread-manager;1"].getService(Ci.nsIThreadManager),
 		$timers = require("sdk/timers"),
 		{xhr_tools:$xhr} = require('./xhr');

exports.getIp = function(host, cb){
	DnsService.asyncResolve(host, 0, {
		onLookupComplete: function(request, record, status) {
			if (!components.isSuccessCode(status)) {
				cb(null, status);
			} else {
				cb(record.getNextAddrAsString(), null);
			}				
		}
	}, ThreadManager.currentThread);
};

// Web Console Api
// conf.host {String} - http://localhost:4445
function WCAPIClient(host){
	this.host = host;
	this.isLock = false;
	this.setFileCheckConfig({
		scan_timeout_ms: 10000,
		cure: false,
		heuristic_analysis: true,
		packer_max_level: 8,
		archive_max_level: 8,
		mail_max_level: 8,
		container_max_level: 8,
		max_compression_ratio: 500	
	});
}
WCAPIClient.prototype.apiRequest = {
	whoami: '/api/10.2/empty',
	login: '/api/10.2/login',
	logout: '/api/10.2/logout',
	categorizeUrl: '/api/10.2/categorize_url',
	categorizeIp: '/api/10.2/categorize_address',
	cloudCheckUrl: '/api/10.2/cloud_check_url',
	scanRequest: '/api/10.2/scan_request',
	scanEndpoint: '/api/10.2/scan_endpoint/',
};
WCAPIClient.prototype.whoami = function(cb){
	return $xhr.get(this.host + this.apiRequest.whoami, null, function(d, r){
		cb(d, r);
	}, 'json');
};
WCAPIClient.prototype.login = function(user, password, cb){
	return $xhr.send('POST', this.host + this.apiRequest.login, {
		'content-type': 'application/json',
	}, JSON.stringify({
		user: user,
		password: password
	}), cb, true);
};
WCAPIClient.prototype.logout = function(cb){
	return $xhr.get(this.host + this.apiRequest.logout, null, function(d, r){
		cb(d, r);
	}, 'json');
};
WCAPIClient.prototype.categorizeUrl = function(url, cb){
	return $xhr.send('POST', this.host + this.apiRequest.categorizeUrl, {
		'content-type': 'application/json',
	}, JSON.stringify({
		url: url
	}), cb, true);
};
WCAPIClient.prototype.categorizeIp = function(ip, cb){
	return $xhr.send('POST', this.host + this.apiRequest.categorizeIp, {
		'content-type': 'application/json',
	}, JSON.stringify({
		address: ip
	}), cb, true);
};
WCAPIClient.prototype.cloudReport = function(url, cb){
	return $xhr.send('POST', this.host + this.apiRequest.cloudCheckUrl, {
		'content-type': 'application/json',
	}, JSON.stringify({
		url: url
	}), cb, true);
}
WCAPIClient.prototype.checkFile = function(data, cb, fb){
	return $xhr.send('POST', this.host + this.apiRequest.scanRequest, {
		'content-type': 'application/json',
	}, this._fileCheckConf, function(d, r){
		if(d.endpoint){
			$xhr.send('POST', this.host + this.apiRequest.scanEndpoint + d.endpoint, {
				'content-type': 'application/json',
			}, data, function(d, r){
				var code = d && d.error;

				if(code){
					fb && fb(code);
				}else{
					cb && cb(d, r);	
				}
			}, true);
		}else{
			fb && fb(d && d.error && d.error.code);
		}
	}.bind(this), true);
}
WCAPIClient.prototype.setFileCheckConfig = function(conf){
	this._fileCheckConf = JSON.stringify(conf);
};
WCAPIClient.prototype.checkFile2 = function(data, cb){
	var _currentRequest = $xhr.send('POST', this.host + this.apiRequest.scanRequest, {
		'content-type': 'application/json',
	}, this._fileCheckConf, function(d, r){
		if(d.endpoint){
			_currentRequest = $xhr.send('POST', this.host + this.apiRequest.scanEndpoint + d.endpoint, {
				'content-type': 'application/json',
			}, data, cb, true);
		}else{
			cb && cb(d); //  && d.error && d.error.code
		}
	}.bind(this), true);

	return _currentRequest;
}
WCAPIClient.prototype.setHost = function(url){
	this.host = 'http://' + (url.trim().split('/')[2]);
	this.isLock = false;
	return this.host;
}
WCAPIClient.prototype.monitorHost = function(){
	this._interval && $timers.clearInterval(this._interval);
	this._interval = $timers.setInterval(function(){
		this.whoami(function(d){
			if(d && !d.error){
				$timers.clearInterval(this._interval);
				this.isLock = false;
			}
		}.bind(this));
	}.bind(this), 10000);
}

var _client = new WCAPIClient();

exports.client = _client;
exports.getClient = function(){
	return _client;
}
/*
Example:
var wc = require('resource://linkcheckerwce/lib/wc_api.js')
wc.getIp('www.yandex.ru',function(ip){
	console.log('IP %s', ip);
}, function(code){
	console.log('TROUBLES %s', code);
})

*/