var 	pageMods = require("sdk/page-mod"),
		MozData = require("sdk/self").data,
		MozTabs = require("sdk/tabs"),
		Popup = require('./popup_dialog'),
		_ = require("sdk/l10n").get,
		BlockManager = require('./block_manager'), // BlockManager.blocks <- BlockService
		BaseManager =require('./base_manager'),
		MozStorage = require("sdk/simple-storage"),
		TabManager = require('./tab_manager'),
		$scanCore = require('./scan_core'),
		$timers = require('sdk/timers'),
		CheckLink = $scanCore.CheckLink;


var		_WORKERS = {};		
exports.workers = _WORKERS;

exports.init = function(){
	var pageMod = pageMods.PageMod({
		include: ['http://*', 'https://*'],
		// exclude: 'http://online.drweb.com/result/*', // don't inject at ccheck url window (Since FFox32!)
		attachTo: 'top', //// to main document (not subframes)
		contentScriptFile: [MozData.url("selector/content.js")],
		contentStyleFile: MozData.url("selector/content.css"),
		contentScriptWhen: 'start',
		onAttach: function(worker){
			var tabId = worker.tab.id;
			_WORKERS[tabId] = worker;
			TabManager.cache.updateTabURL(tabId, worker.tab.url);

			worker.port.on('getBlocks', function(){
				var 	$pref = MozStorage.storage.preference,
					 	blocks = TabManager.cache.getBlocks(tabId),
						tabHost = TabManager.cache.getHost(tabId) || '',
						tabHost2 = tabHost.replace(/^www\./, ''),
						flashStatus = BlockManager.blocks.getAdsException(tabHost, true), // detect if settings for this domain exist
						adsStatus = BlockManager.blocks.getAdsException(tabHost),
						blockFlash = false,
						blockAds = false;
				
				if(flashStatus == 2 || (flashStatus == 4  /*&& $pref.flash_enabled*/)){
					blockFlash = true;
				}
				
				if(adsStatus == 2 || (adsStatus == 4  /*&& $pref.ads_enabled*/)){
					blockAds = true;
				}
				
				worker.port.emit("cleanBlocks", { 
					blocks: blocks,
					manual: BlockManager.blocks.get(tabHost2), 
					google_check: $pref.google_check,
					vkontakte_check: $pref.vkontakte_check,
					facebook_check: $pref.facebook_check,
					specSocialBlocks: BaseManager.socialBlocks.filter(function(item){return item.baseURL == tabHost2;}),
					blockFlash: blockFlash,
					blockAds: blockAds,
					adsPatterns: blockAds && BaseManager.blockHash['Ads'] ? BaseManager.blockHash['Ads'].map(function(obj){return obj.pattern;}) : [],
				});
				
				if(MozTabs.activeTab.id == tabId){ // If active tab
					Popup.changeBadge(tabId, TabManager.getBlockRequests(tabId).length);
				}
			});
			
			worker.on('detach', function(){ // эквивалент перезагрузки страницы
				_WORKERS[tabId] = null;
				TabManager.cache.remove(tabId); // flush cache
			});
			
			worker.port.on('clearBlocks', function(data){
				BlockManager.blocks.remove(data.host, data.selector);	
			});
			worker.port.on('saveBlocks', function(data){
				// BlockManager.blocks.save(data.blocks);
				BlockManager.blocks.save(data.host, data.blocks);
			});
			worker.port.on('clearBlockRule', function(data){
				BlockManager.blocks.removeRule(data.host, data.domString);
			});
			
			worker.port.on('checkURL', function(data){
				var 	abortTimeout;
					 	_data = {};

				var linkChecker = new CheckLink(data.url, {
					redirect: function(url){
	     				_data.url = url;
	     			},
					error: function(mes, code){
						_data.message = mes;
						_data.code = code;
					},
					ipCheck: function(cat, own){
						_data.categories = cat.map(function(key){ return $scanCore.categoryTitle[key]||key;});
						_data.legal_url = own;
						$timers.clearTimeout(abortTimeout);
					},
					urlCheck: function(cat, own){
						_data.categories = cat.map(function(key){ return $scanCore.categoryTitle[key]||key;});
						_data.legal_url = own;
						$timers.clearTimeout(abortTimeout);
					}
				});
				linkChecker.wf.add(function(next,conf, scope){ // complete task
					worker.port.emit('check-result', _data);
					_linkChecker = null;
				});
				// how to destroy:
				
				abortTimeout = $timers.setTimeout(function(){
					linkChecker && linkChecker.abort && linkChecker.abort();
					linkChecker = null;
				}, 11000);
			});
		},
		contentScriptOptions: {
			backImage: MozData.url('selector/back1.gif'),
			toolbox_css: MozData.url('selector/toolbox.css'),
			img: {
				logo: MozData.url('selector/logo.png'),
				warning: MozData.url('selector/warning_link.png'),
				ok: MozData.url('selector/ok_link.png'),
				error: MozData.url('selector/error_link.png'),
				threat: MozData.url('selector/threat_link.png'),
			},
			liter: { 
				panel_description: _('panel_description'),
				save: _('save'),
				cancel: _('cancel'),
				restore: _('restore'),
				remove: _('remove'),
				lang: _('lang'),
				informer_message: _('reload_page'),
				informer_reload_now: _('reload_now'),
				informer_reload_later: _('reload_later'),
				go: _('go'),
				scanning: _('scanning'),
				close: _('close'),
				manualUnBlockingText: _('manualUnBlockingText'),
			},
			categories: {
				INFECTION_SOURCE: _('UC_INFECTION_SOURCE'),
				NOT_RECOMMENDED: _('UC_NOT_RECOMMENDED'),
				ADULT_CONTENT: _('UC_ADULT_CONTENT'),
				VIOLENCE: _('UC_VIOLENCE'),
				WEAPONS: _('UC_WEAPONS'),
				GAMBLING: _('UC_GAMBLING'),
				DRUGS: _('UC_DRUGS'),
				OBSCENE_LANGUAGE: _('UC_OBSCENE_LANGUAGE'),
				CHATS: _('UC_CHATS'),
				TERRORISM: _('UC_TERRORISM'),
				FREE_EMAIL: _('UC_FREE_EMAIL'),
				SOCIAL_NETWORKS: _('UC_SOCIAL_NETWORKS'),
				OWNERS_NOTICE: _('UC_OWNERS_NOTICE'),
				BLACK_LIST: _('UC_BLACK_LIST'),
			}
		}
	});
};

// @param {Array | String} host
exports.showInformerForHost = function(host){
	var 	tabHost,
			hostHash = {};
	
	if(Array.isArray(host)){
		host.forEach(function(hostStr){
			hostHash[hostStr] = 1;
		});
	}else{
		hostHash[host] = 1;
	}		
	
	for(var tabId in _WORKERS){
		if(_WORKERS.hasOwnProperty(tabId) && (worker = _WORKERS[tabId])){
			tabHost = TabManager.cache.getHost(worker.tab.id);
			
			if(tabHost && hostHash[tabHost]){
				worker.port.emit('updateExtProperties', {
					group: 'informAllTabs'
				});
			}
		}
	}
	hostHash = null;
};
// @param {String} group
// @param {Object} updatedSettings
exports.refreshExtProperties = function(group, updatedSettings){
	var settings = {
		group: group,
		value: updatedSettings
	};
	
	for(var tabId in _WORKERS){
		if(_WORKERS.hasOwnProperty(tabId) && _WORKERS[tabId]){
			_WORKERS[tabId].port.emit('updateExtProperties', settings);
		}
	}
};

exports.toggleManualPanel = function(tabId){
	var worker = _WORKERS[tabId];
		
	if(worker){
		worker.port.emit('toggleManualPanel', {});
	}
}