// const XMLHttpRequest = require("sdk/net/xhr").XMLHttpRequest; // fall with error exception at FF39
const {Cc, Ci} = require("chrome");
const XMLHttpRequest = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"];

exports.xhr_tools = {
	_isJsonMIME: /application\/json/,
	_tryBack: function(cb){
		try{
			return cb();
		}catch(e){
			return null;
		}
	},
	send: function(method, url, headers, body, oncomplete, forceJSON) {
		var xhr = new XMLHttpRequest();
		xhr.onreadystatechange = function(data){
			if (xhr.readyState == 4){
				var 	contentType = xhr.getResponseHeader('Content-Type'),
						response;

				if(this._isJsonMIME.test(contentType) && xhr.responseText || forceJSON){
					response = this._tryBack(function(){
						return JSON.parse(xhr.responseText);
					});
				}else{
					response = xhr.responseText;
				}

                oncomplete(response, xhr);
            } //else  Load continue
		}.bind(this);
		//xhr.overrideMimeType('application/json');
		xhr.open(method, url, true);
		if (headers) {
			for (var header in headers) {
				if (headers.hasOwnProperty(header)) {
					xhr.setRequestHeader(header, headers[header]);
				}
			}
		}
		xhr.send(body);
		return xhr;
	},
	_3986map: {
		'!': 	'%21',
		'*': 	'%2A',
		'\'': 	'%27',
		'(': 	'%28',
		')': 	'%29',
	},
	_3986backmap: {
		'%21': '!',
		'%2A': '*',
		'%27': '\'',
		'%28': '(',
		'%29': ')',
	},
	toRfc3986: function(val){
		return encodeURIComponent(val).replace(/[\!\*\'\(\)]/g, function(m){
			return this._3986map[m];
		}.bind(this));
	},
	fromRfc3986: function(val){
		return decodeURIComponent(val.replace(/(%21|%2A|%27|%28|%29)/g, function(m){
			return this._3986backmap[m];
		}.bind(this)));
	},
	urlEncode: function(obj){
		var list = [];
		for(key in obj){
			list.push(this.toRfc3986(key) + '=' + this.toRfc3986(obj[key]));
		}
		return list.join('&');
	},
	get: function(url, params, oncomplete, resFormat){
		return this.send('GET', url + (params ? '?' + this.urlEncode(params) : ''), null, null, oncomplete || function(){}, resFormat == 'json');
	},
	post: function(url, params, oncomplete, resFormat){
		return this.send('POST', url, {
			'content-type': 'application/x-www-form-urlencoded;charset=utf-8'
		}, this.urlEncode(params), oncomplete || function(){}, resFormat == 'json');
	},
	// request to detect final url (after redirection) and ip (by chrome api)
	emptyGet: function(url, cb){
		var 	xhr = new XMLHttpRequest(),
				_status,
				_statusText,
				_responseUrl;

		xhr.onreadystatechange = function(e){
			if(xhr.readyState == 2){
				_status = xhr.status;
				_statusText = xhr.statusText;
				_responseUrl = xhr.responseURL;
				xhr.abort();
			}else if(xhr.readyState == 4){ // fires from network troubles and from abort() when state == 2
				cb && cb(_status || xhr.status, _responseUrl, _statusText || xhr.statusText);
			}
		};
		xhr.open('GET', url, true);
		xhr.send(null);
	},
	downloadFile: function(url, cb){
		var xhr = new XMLHttpRequest(); 
		xhr.open('GET', url, true); 
		xhr.responseType = 'arraybuffer';
		xhr.onreadystatechange = function(){ 
			if (xhr.readyState == 4) {
				if (cb){
					cb(xhr.response, xhr);	
				} 
			}
		};
		xhr.send(null);
	},
};